# -*- coding: utf-8 -*-
# mypy: ignore-errors
#
# Copyright © 2025 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
#
# This program is a free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# You can get a copy of GNU General Public License along this program
# But you can always get it from http://www.gnu.org/licenses/gpl.txt
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

from __future__ import annotations

from gi.repository import Gtk as gtk

from pyglossary.sort_keys import defaultSortKeyName, namedSortKeyList

from .utils import HBox, pack

__all__ = ["SortOptionsBox"]


# log = logging.getLogger("pyglossary")


sortKeyNameByDesc = {sk.desc: sk.name for sk in namedSortKeyList}
sortKeyNames = [sk.name for sk in namedSortKeyList]


# Note: RadioButton does not exist in Gtk 4.0,
# you have to use CheckButton with group= arg or set_group() method

# Note: Gtk.CheckButton is not a subclass of Gtk.Button!


class SortOptionsBox(gtk.Box):
	def __init__(self, mainWin: gtk.Window) -> None:
		gtk.Box.__init__(self, orientation=gtk.Orientation.VERTICAL)
		self.mainWin = mainWin
		###
		self.set_spacing(5)
		###
		hbox = HBox(spacing=5)
		sortCheck = gtk.CheckButton(label="Sort entries by")
		sortKeyCombo = gtk.ComboBoxText()
		for sk in namedSortKeyList:
			sortKeyCombo.append_text(sk.desc)
		sortKeyCombo.set_active(sortKeyNames.index(defaultSortKeyName))
		sortKeyCombo.set_sensitive(False)
		# sortKeyCombo.connect("changed", self.sortKeyComboChanged)
		self.sortCheck = sortCheck
		self.sortKeyCombo = sortKeyCombo
		sortCheck.connect("toggled", self.onSortCheckToggled)
		pack(hbox, sortCheck)
		pack(hbox, sortKeyCombo)
		pack(self, hbox)
		###
		hbox = self.encodingHBox = HBox(spacing=5)
		encodingCheck = self.encodingCheck = gtk.CheckButton(label="Sort Encoding")
		encodingEntry = self.encodingEntry = gtk.Entry()
		encodingEntry.set_text("utf-8")
		encodingEntry.set_width_chars(15)
		pack(hbox, gtk.Label(label="    "))
		pack(hbox, encodingCheck)
		pack(hbox, encodingEntry)
		pack(self, hbox)
		encodingCheck.set_active(True)
		###
		hbox = self.localeHBox = HBox(spacing=5)
		localeEntry = self.localeEntry = gtk.Entry()
		localeEntry.set_width_chars(15)
		localeEntry.set_text("latin")
		pack(hbox, gtk.Label(label="    "))
		self.localeCheck = gtk.CheckButton(
			label="Sort Locale",
			group=self.encodingCheck,
		)
		pack(hbox, self.localeCheck)
		pack(hbox, localeEntry)
		pack(self, hbox)
		###
		checkBoxSizeGroup = gtk.SizeGroup(mode=gtk.SizeGroupMode.HORIZONTAL)
		checkBoxSizeGroup.add_widget(encodingCheck)
		###
		self.show()

	def onSortCheckToggled(self, *_args: object) -> None:
		sort = self.sortCheck.get_active()
		self.sortKeyCombo.set_sensitive(sort)
		self.encodingHBox.set_sensitive(sort)

	def updateWidgets(self) -> None:
		convertOptions = self.mainWin.convertOptions
		sort = bool(convertOptions.get("sort", False))
		self.sortCheck.set_active(sort)
		self.sortKeyCombo.set_sensitive(sort)
		self.encodingHBox.set_sensitive(sort)
		self.localeHBox.set_sensitive(sort)

		if "sortEncoding" in convertOptions:
			self.encodingCheck.set_active(True)
			self.encodingEntry.set_text(convertOptions["sortEncoding"])

		sortKeyName = convertOptions.get("sortKeyName")
		if sortKeyName:
			sortKeyName, _, localeName = sortKeyName.partition(":")
			if sortKeyName:
				self.sortKeyCombo.set_active(sortKeyNames.index(sortKeyName))
			self.localeEntry.set_text(localeName)
			if localeName:
				self.localeCheck.set_active(True)

	def applyChanges(self) -> None:
		convertOptions = self.mainWin.convertOptions
		sort = self.sortCheck.get_active()
		if not sort:
			for param in ("sort", "sortKeyName", "sortEncoding"):
				if param in convertOptions:
					del convertOptions[param]
			return

		convertOptions["sort"] = True

		sortKeyDesc = self.sortKeyCombo.get_active_text()
		sortKeyName = sortKeyNameByDesc[sortKeyDesc]
		if self.localeCheck.get_active():
			sortLocale = self.localeEntry.get_text()
			if sortLocale:
				sortKeyName = f"{sortKeyName}:{sortLocale}"
				if "sortEncoding" in convertOptions:
					del convertOptions["sortEncoding"]
		elif self.encodingCheck.get_active():
			convertOptions["sortEncoding"] = self.encodingEntry.get_text()

		convertOptions["sortKeyName"] = sortKeyName



import plistlib
import sys
import unittest
from os.path import abspath, dirname, join

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)

from glossary_v2_test import TestGlossaryBase

from pyglossary.glossary_v2 import ConvertArgs, Glossary


class TestGlossaryAppleDict(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		hashDict = {
			"appledict-src/002-no-morphology-v3.txt": "d8086fe8",
			"appledict-src/002-no-morphology-v3a/002-no-morphology-v3a.css": "6818c1e5",
			"appledict-src/002-no-morphology-v3a/002-no-morphology-v3a.plist": "706d1d9c",
			"appledict-src/002-no-morphology-v3a/002-no-morphology-v3a.xml": "707994d6",
			"appledict-src/002-no-morphology-v3a/Makefile": "ecd42350",
		}
		self.dataFileCRC32.update(hashDict)

	def comparePlist(self, fpath1, fpath2):
		with open(fpath1, "rb") as file:
			data1 = plistlib.loads(file.read())
		with open(fpath2, "rb") as file:
			data2 = plistlib.loads(file.read())
		self.assertEqual(data1, data2)

	def test_tabfile_without_morpho_to_appledict_source(self):
		self.glos = Glossary()

		inputName = "002-no-morphology-v3"
		outputName = "002-no-morphology-v3a"
		inputFilepath = self.downloadFile(f"appledict-src/{inputName}.txt")
		outputDirPath = self.newTempFilePath(f"{outputName}")

		expectedFiles = {
			name: self.downloadFile(f"appledict-src/{outputName}/{name}")
			for name in [
				f"{outputName}.xml",
				f"{outputName}.css",
				"Makefile",
			]
		}

		result = self.glos.convert(
			ConvertArgs(
				inputFilename=inputFilepath,
				outputFilename=outputDirPath,
				inputFormat="Tabfile",
				outputFormat="AppleDict",
			)
		)
		self.assertIsNotNone(result)
		self.assertEqual(result, outputDirPath)

		for fname, fpath in expectedFiles.items():
			self.compareTextFiles(
				join(outputDirPath, fname),
				fpath,
			)

		self.comparePlist(
			join(outputDirPath, f"{outputName}.plist"),
			self.downloadFile(f"appledict-src/{outputName}/{outputName}.plist"),
		)


if __name__ == "__main__":
	unittest.main()



import os
import sys
import unittest
from os.path import abspath, dirname

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)

from glossary_v2_test import TestGlossaryBase


class TestGlossaryGetttestPo(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"100-en-fa.po": "694de186",
				"100-en-fa.po.txt": "f0c3ea53",
			},
		)
		os.environ["CALC_FILE_SIZE"] = "1"

	def convert_txt_po(self, fname, fname2, **convertArgs):
		self.convert(
			f"{fname}.txt",
			f"{fname}-2.po",
			compareText=f"{fname2}.po",
			**convertArgs,
		)

	def convert_po_txt(self, fname, fname2, **convertArgs):
		self.convert(
			f"{fname}.po",
			f"{fname}-2.txt",
			compareText=f"{fname2}.txt",
			**convertArgs,
		)

	def test_convert_txt_po_1(self):
		self.convert_txt_po("100-en-fa", "100-en-fa")

	# TODO
	def test_convert_po_txt_1(self):
		self.convert_po_txt(
			"100-en-fa",
			"100-en-fa.po",
			infoOverride={"input_file_size": None},
		)


if __name__ == "__main__":
	unittest.main()



import unittest

from glossary_v2_test import TestGlossaryBase


class TestGlossaryJSON(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"004-bar.json": "7e4b2663",
				"100-en-de-v4.json": "6a20c6f6",
				"100-en-fa.json": "8d29c1be",
				"100-ja-en.json": "fab2c106",
			},
		)

	def convert_txt_json(self, fname):
		self.convert(
			f"{fname}.txt",
			f"{fname}-2.json",
			compareText=f"{fname}.json",
		)

	def test_convert_txt_json_0(self):
		self.convert_txt_json("004-bar")

	def test_convert_txt_json_1(self):
		self.convert_txt_json("100-en-fa")

	def test_convert_txt_json_2(self):
		self.convert_txt_json("100-en-de-v4")

	def test_convert_txt_json_3(self):
		self.convert_txt_json("100-ja-en")


if __name__ == "__main__":
	unittest.main()



from __future__ import annotations

import re
import unicodedata
from collections import defaultdict
from io import BytesIO
from typing import TYPE_CHECKING, NamedTuple, cast

from lxml import etree as ET

from pyglossary.core import log

from .pinyin import convert
from .summarize import summarize
from .util import get_chinese_references

if TYPE_CHECKING:
	from collections.abc import Callable, Sequence

	from pyglossary.lxml_types import T_htmlfile


__all__ = [
	"Article",
	"create_render_definition",
	"parse_line_simp",
	"parse_line_trad",
	"render_article",
	"render_definition_no_links",
	"render_definition_with_links",
	"render_syllables",
	"render_syllables_color",
	"render_syllables_no_color",
]


_re_line = re.compile(r"^([^ ]+) ([^ ]+) \[([^\]]+)\] /(.+)/$")

_COLORS = {
	"": "black",
	"1": "red",
	"2": "orange",
	"3": "green",
	"4": "blue",
	"5": "black",
}


def parse_line_trad(line: str) -> tuple[str, str, str, list[str]] | None:
	line = line.strip()
	match = _re_line.match(line)
	if match is None:
		return None
	trad, simp, pinyin, eng = match.groups()
	pinyin = pinyin.replace("u:", "ü")
	return trad, simp, pinyin, eng.split("/")


def parse_line_simp(line: str) -> tuple[str, str, str, list[str]] | None:
	line = line.strip()
	match = _re_line.match(line)
	if match is None:
		return None
	trad, simp, pinyin, eng = match.groups()
	pinyin = pinyin.replace("u:", "ü")
	return simp, trad, pinyin, eng.split("/")


class Article(NamedTuple):
	first: str
	second: str
	pinyin: str
	eng: list[str]

	def names(self) -> list[str]:
		if self.first == self.second:
			return [self.first]
		return [self.first, self.second]

	def definition_summaries(self) -> list[str]:
		return list(map(summarize, self.eng))


def render_syllables(
	hf: T_htmlfile,
	syllables: Sequence[str],
	tones: Sequence[str],
	color: bool = True,
) -> None:
	if color and len(syllables) != len(tones):
		log.warning(f"unmatched tones: {syllables=}, {tones=}")
		color = False

	colors = _COLORS if color else defaultdict(lambda: "")

	with hf.element("div", style="display: inline-block"):
		for index, syllable in enumerate(syllables):
			tone = tones[index] if len(syllables) == len(tones) else ""
			with hf.element("font", color=colors[tone]):
				if index > 0:
					if syllable[0].isupper() and tone:
						# Add a space before a capitalized syllable.
						hf.write(" ")
					elif unicodedata.normalize("NFD", syllable[0])[0] in "aeiou" and tone:
						# Add an apostrophe before a vowel.
						hf.write("'")
				hf.write(syllable)


def render_syllables_no_color(
	hf: T_htmlfile,
	syllables: Sequence[str],
	tones: Sequence[str],
) -> None:
	render_syllables(
		hf=hf,
		syllables=syllables,
		tones=tones,
		color=False,
	)


def render_syllables_color(
	hf: T_htmlfile,
	syllables: Sequence[str],
	tones: Sequence[str],
) -> None:
	render_syllables(
		hf=hf,
		syllables=syllables,
		tones=tones,
		color=True,
	)


def render_definition_no_links(hf: T_htmlfile, definition: str) -> None:
	with hf.element("li"):
		hf.write(definition)


def render_definition_with_links(
	hf: T_htmlfile,
	traditional_title: bool,
	definition: str,
) -> None:
	chinese_refs = get_chinese_references(definition)
	get_chinese = (
		(lambda match: match.trad) if traditional_title else (lambda match: match.simp)
	)

	match = None
	with hf.element("li"):
		try:
			match = next(chinese_refs)
			hf.write(definition[0 : match.start])
			with hf.element("a", href=f"bword://{get_chinese(match)}"):
				hf.write(match.text)

		except StopIteration:
			hf.write(definition)
			return

		for next_match in chinese_refs:
			hf.write(definition[match.end : next_match.start])
			match = next_match
			with hf.element(
				"a",
				href=f"bword://{get_chinese(match)}",
			):
				hf.write(match.text)

		hf.write(definition[match.end :])


def create_render_definition(
	traditional_title: bool,
	link_references: bool,
) -> Callable[[T_htmlfile, str], None]:
	if not link_references:
		return render_definition_no_links

	def func(hf: T_htmlfile, definition: str) -> None:
		render_definition_with_links(
			hf=hf,
			definition=definition,
			traditional_title=traditional_title,
		)

	return func


# @lru_cache(maxsize=128)
def _convert_pinyin(pinyin: str) -> tuple[Sequence[str], Sequence[str]]:
	return tuple(zip(*map(convert, pinyin.split()), strict=False))  # type: ignore


def render_article(
	render_syllables: Callable[[T_htmlfile, str], None],
	render_definition: Callable[[T_htmlfile, str], None],
	article: Article,
) -> tuple[list[str], str]:
	names = article.names()

	# pinyin_tones = [convert(syl) for syl in pinyin.split()]
	pinyin_list, tones = _convert_pinyin(article.pinyin)

	f = BytesIO()
	with ET.htmlfile(f, encoding="utf-8") as hf_:  # noqa: PLR1702
		hf = cast("T_htmlfile", hf_)
		with hf.element("div", style="border: 1px solid; padding: 5px"):
			with hf.element("div"):
				with hf.element("big"):
					render_syllables(hf, names[0], tones)

				if len(names) > 1:
					hf.write("\xa0/\xa0")  # "\xa0" --> "&#160;" == "&nbsp;"
					render_syllables(hf, names[1], tones)

				hf.write(ET.Element("br"))

				with hf.element("big"):
					render_syllables(hf, pinyin_list, tones)

			with hf.element("div"):
				with hf.element("ul"):
					for defn in article.eng:
						render_definition(hf, defn)

	return names, f.getvalue().decode("utf-8")



import sys
import unittest
from os.path import abspath, dirname

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)

from pyglossary import __version__
from pyglossary.logger import format_exception
from pyglossary.plugins.ebook_kobo import (
	description,
	enable,
	extensionCreate,
	extensions,
	kind,
	lname,
	name,
	optionsProp,
	singleFile,
	website,
	wiki,
)
from pyglossary.plugins.stardict import (
	Reader,
	description,
	enable,
	extensionCreate,
	extensions,
	kind,
	lname,
	name,
	optionsProp,
	singleFile,
	website,
	wiki,
)
from pyglossary.plugins.tabfile import (
	Writer,
	description,
	enable,
	extensionCreate,
	extensions,
	kind,
	lname,
	name,
	optionsProp,
	singleFile,
	website,
	wiki,
)
from pyglossary.sort_keys import LocaleNamedSortKey


class TestDummy(unittest.TestCase):
	def test_test(self):
		pass



# -*- coding: utf-8 -*-
from __future__ import annotations

from typing import Literal, NotRequired, Required, TypeAlias

from typing_extensions import TypedDict


class DefinitionString(TypedDict):
	type: Literal["text"]
	text: str


class DefinitionImage(TypedDict, total=False):
	type: Required[Literal["image"]]
	path: Required[str]
	width: int
	height: int
	title: str
	alt: str
	description: str
	pixelated: bool
	imageRendering: Literal["auto", "pixelated", "crisp=edges"]
	appearance: Literal["auto", "monochrome"]
	background: bool
	collapsed: bool
	collapsible: bool


class DefinitionStructContent(TypedDict):
	type: Literal["structured-content"]
	content: StructuredContent


DefinitionObj: TypeAlias = DefinitionString | DefinitionImage | DefinitionStructContent
YomichanDefinition: TypeAlias = str | DefinitionObj | tuple[str, list[str]]


DataAttributes: TypeAlias = dict[str, str]
StyleAttributes: TypeAlias = dict[str, str | int]


class EmptyTagStructContent(TypedDict):
	tag: Literal["br"]
	data: NotRequired[DataAttributes]


class GenericContainerStructContent(TypedDict, total=False):
	tag: Required[Literal["ruby", "rt", "rp", "table", "thead", "tbody", "tfoot", "tr"]]
	content: StructuredContent
	data: DataAttributes
	lang: str


class TableStructContent(TypedDict, total=False):
	tag: Required[Literal["td", "th"]]
	content: StructuredContent
	data: DataAttributes
	colSpan: int
	rowSpan: int
	style: StyleAttributes
	lang: str


class StylishContainerStructContent(TypedDict, total=False):
	tag: Required[Literal["span", "div", "ol", "ul", "li", "details", "summary"]]
	content: StructuredContent
	data: DataAttributes
	style: StyleAttributes
	# Hover text
	title: str
	open: bool
	lang: str


class ImageTagStructContent(TypedDict, total=False):
	tag: Required[Literal["img"]]
	data: DataAttributes
	path: Required[str]
	width: float
	height: float
	# Hover text
	title: str
	alt: str
	description: str
	pixelated: bool
	imageRendering: Literal["auto", "pixelated", "crisp=edges"]
	appearance: Literal["auto", "monochrome"]
	background: bool
	collapsed: bool
	collapsible: bool
	verticalAlign: Literal[
		"baseline", "sub", "super", "text-top", "text-bottom", "middle", "top", "bottom"
	]  # noqa: E501
	border: str
	borderRadius: str
	# The units for the width and height
	sizeUnits: Literal["px", "em"]


class LinkTagStructContent(TypedDict, total=False):
	tag: Required[Literal["a"]]
	content: StructuredContent
	href: Required[str]
	lang: str


StructuredContentObj: TypeAlias = (
	EmptyTagStructContent
	| GenericContainerStructContent
	| TableStructContent
	| StylishContainerStructContent
	| ImageTagStructContent
	| LinkTagStructContent
)
StructuredContent: TypeAlias = str | list["StructuredContent"] | StructuredContentObj



import unittest

from glossary_v2_test import TestGlossaryBase


class TestGlossaryDictionaryForMIDs(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

	def convert_txt_mids(self, fname, sha1sumDict, **convertArgs):
		outputFname = f"{fname}-2.mids.zip"
		outputFpath = self.newTempFilePath(outputFname)
		# expectedFpath = self.downloadFile(f"{fname}.mids.zip")
		self.convert(
			f"{fname}.txt",
			outputFname,
			**convertArgs,
		)
		self.checkZipFileSha1sum(outputFpath, sha1sumDict)

	def test_convert_txt_mids_1(self):
		sha1sumDict = {
			"DictionaryForMIDs.properties": "4260a87d6cdd55622dcfe395880bc913f96102b8",
			"directory1.csv": "1f1ab12b107608a1513254fff3c323bbcdfbd5cf",
			"index1.csv": "494268da410c520e56142b47610f6bbcfd53c79f",
			"searchlist.csv": "4f4513d1550436e867e1a79dbd073a7e5bb38e32",
		}
		self.convert_txt_mids("100-en-fa", sha1sumDict)


if __name__ == "__main__":
	unittest.main()



# -*- coding: utf-8 -*-
# mypy: ignore-errors
#
# Copyright © 2025 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
#
# This program is a free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# You can get a copy of GNU General Public License along this program
# But you can always get it from http://www.gnu.org/licenses/gpl.txt
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

from __future__ import annotations

from typing import TYPE_CHECKING

from gi.repository import GLib as glib
from gi.repository import Gtk as gtk

if TYPE_CHECKING:
	from collections.abc import Callable

	from gi.repository import Gio as gio

__all__ = ["BrowseButton"]


class BrowseButton(gtk.Button):
	def __init__(
		self,
		setFilePathFunc: Callable[[str], None],
		label: str = "Browse",
		actionSave: bool = False,
		title: str = "Select File",
	) -> None:
		gtk.Button.__init__(self)

		self.set_label(label)
		# TODO: self.set_icon_name
		# self.set_image(gtk.Image.new_from_icon_name(
		# 	"document-save" if actionSave else "document-open",
		# 	gtk.IconSize.BUTTON,
		# ))

		self.actionSave = actionSave
		self.setFilePathFunc = setFilePathFunc
		self.title = title

		self.connect("clicked", self.onClick)

	def onFiledialogOpen(
		self,
		filedialog: gtk.FileDialog,
		task: gio.Task,
	) -> None:
		try:
			file = filedialog.open_finish(task)
		except glib.GError:
			return
		if file is None:
			return
		self.setFilePathFunc(file.get_path())

	def onFiledialogSave(
		self,
		filedialog: gtk.FileDialog,
		task: gio.Task,
	) -> None:
		try:
			file = filedialog.save_finish(task)
		except glib.GError:
			return
		if file is None:
			return
		self.setFilePathFunc(file.get_path())

	def onClick(self, _widget: gtk.Widget) -> None:
		dialog = gtk.FileDialog.new()
		dialog.set_title(self.title)
		# dialog.set_initial_folder(dir_name)
		if self.actionSave:
			dialog.save(
				parent=self.get_root(),
				cancellable=None,
				callback=self.onFiledialogSave,
			)
		else:
			dialog.open(
				parent=self.get_root(),
				cancellable=None,
				callback=self.onFiledialogOpen,
			)



import os
import unittest

from glossary_v2_test import TestGlossaryBase


class TestGlossarySQL(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"100-en-fa.txt": "f5c53133",
				"100-en-fa.txt-v2.sql": "70cd0514",
			},
		)
		os.environ["CALC_FILE_SIZE"] = "1"

	def convert_txt_sql(self, fname, fname2, **convertArgs):
		self.convert(
			f"{fname}.txt",
			f"{fname}-2.sql",
			compareText=f"{fname2}.sql",
			**convertArgs,
		)

	def test_convert_txt_sql_1(self):
		self.convert_txt_sql(
			"100-en-fa",
			"100-en-fa.txt-v2",
		)


if __name__ == "__main__":
	unittest.main()



import logging
import os
import sys
import unittest
from os.path import abspath, dirname

rootDir = dirname(dirname(dirname(abspath(__file__))))
sys.path.insert(0, rootDir)

from glossary_errors_test import TestGlossaryErrors
from glossary_test import testCacheDir

from pyglossary.glossary import Glossary


class TestGlossarySecurity(TestGlossaryErrors):
	def __init__(self, *args, **kwargs):
		TestGlossaryErrors.__init__(self, *args, **kwargs)
		self.mockLog.setLevel(logging.INFO)

	def test_convert_1(self):
		glos = Glossary()
		res = glos.convert(
			inputFilename="os.system('abcd')",
			outputFilename="os.system('abcd -l')",
		)
		self.assertIsNone(res)
		self.assertLogCritical("Unable to detect output format!")

	def test_convert_2(self):
		glos = Glossary()
		res = glos.convert(
			inputFilename="os.system('abcd');test.txt",
			outputFilename="os.system('abcd -l')",
		)
		self.assertIsNone(res)
		self.assertLogCritical("Unable to detect output format!")

	def test_convert_3(self):
		glos = Glossary()
		res = glos.convert(
			inputFilename="os.system('abcd');test.txt",
			outputFilename="os.system('abcd -l');test.csv",
		)
		self.assertIsNone(res)
		errMsg = (
			"[Errno 2] No such file or directory: "
			f"\"{testCacheDir}{os.sep}os.system('abcd');test.txt\""
		)
		errMsg = errMsg.replace("\\", "\\\\")
		self.assertLogCritical(errMsg)
		self.assertLogCritical(
			"Reading file \"os.system('abcd');test.txt\" failed.",
		)

	def test_convert_4(self):
		glos = Glossary()
		res = glos.convert(
			inputFilename="test.txt\nos.system('abcd')",
			outputFilename="test.csv\nos.system('abcd -l')",
		)
		self.assertIsNone(res)
		self.assertLogCritical("Unable to detect output format!")


if __name__ == "__main__":
	unittest.main()



import logging
import os
import sys
import unittest
from os.path import abspath, dirname, isfile, join, relpath

rootDir = dirname(dirname(dirname(abspath(__file__))))
sys.path.insert(0, rootDir)

from glossary_test import TestGlossaryBase, appTmpDir

from pyglossary.core_test import getMockLogger
from pyglossary.glossary import Glossary
from pyglossary.os_utils import rmtree

__all__ = ["TestGlossaryErrors"]


Glossary.init()


class MyStr(str):
	__slots__ = []


class TestGlossaryErrorsBase(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)
		self.mockLog = getMockLogger()

	def setUp(self):
		TestGlossaryBase.setUp(self)
		self.mockLog.clear()

	def tearDown(self):
		TestGlossaryBase.tearDown(self)
		method = self._testMethodName
		self.assertEqual(0, self.mockLog.printRemainingErrors(method))
		warnCount = self.mockLog.printRemainingwWarnings(method)
		if warnCount > 0:
			print(
				f"Got {warnCount} unhandled warnings "
				f"from {self.__class__.__name__}: {self._testMethodName}\n",
			)

	def assertLogCritical(self, errorMsg):
		self.assertIsNotNone(
			self.mockLog.popLog(
				logging.CRITICAL,
				errorMsg,
			),
			msg=f"did not find critical log {errorMsg!r}",
		)

	def assertLogError(self, errorMsg):
		self.assertIsNotNone(
			self.mockLog.popLog(
				logging.ERROR,
				errorMsg,
			),
			msg=f"did not find error log {errorMsg!r}",
		)

	def assertLogWarning(self, errorMsg):
		self.assertIsNotNone(
			self.mockLog.popLog(
				logging.WARNING,
				errorMsg,
			),
			msg=f"did not find warning log {errorMsg!r}",
		)


def osRoot():
	if os.sep == "\\":
		return "C:\\"
	return "/"


if os.sep == "\\":
	osNoSuchFileOrDir = "[WinError 3] The system cannot find the path specified:"
else:
	osNoSuchFileOrDir = "[Errno 2] No such file or directory:"


class TestGlossaryErrors(TestGlossaryErrorsBase):
	def test_loadPlugins_invalidDir(self):
		path = join(osRoot(), "abc", "def", "ghe")
		Glossary.loadPlugins(path)
		self.assertLogCritical(f"Invalid plugin directory: {path!r}")

	def test_detectInputFormat_err1(self):
		res = Glossary.detectInputFormat(
			filename="",
			formatName="",
		)
		self.assertIsNone(res)
		self.assertLogCritical("Unable to detect input format!")

	def test_detectInputFormat_err2(self):
		res = Glossary.detectInputFormat(
			filename="test.abcd",
			formatName="",
		)
		self.assertIsNone(res)
		self.assertLogCritical("Unable to detect input format!")

	def test_detectInputFormat_err3(self):
		res = Glossary.detectInputFormat(
			filename="test.sql",
			formatName="",
		)
		self.assertIsNone(res)
		self.assertLogCritical("plugin Sql does not support reading")

	def test_detectInputFormat_err4(self):
		res = Glossary.detectInputFormat(
			filename="test",
			formatName="FooBar",
		)
		self.assertIsNone(res)
		self.assertLogCritical("Invalid format 'FooBar'")

	def test_detectInputFormat_ok1(self):
		res = Glossary.detectInputFormat(
			filename="test1.txt.gz",
			formatName="",
		)
		self.assertEqual(res, ("test1.txt.gz", "Tabfile", ""))

	def test_detectInputFormat_ok2(self):
		res = Glossary.detectInputFormat(
			filename="test2.txt.zip",
			formatName="",
		)
		self.assertEqual(res, ("test2.txt", "Tabfile", "zip"))

	def test_detectOutputFormat_err1(self):
		res = Glossary.detectOutputFormat(
			filename="",
			formatName="",
			inputFilename="",
		)
		self.assertIsNone(res)
		self.assertLogCritical("Invalid filename ''")

	def test_detectOutputFormat_err2(self):
		res = Glossary.detectOutputFormat(
			filename="test",
			formatName="FooBar",
			inputFilename="",
		)
		self.assertIsNone(res)
		self.assertLogCritical("Invalid format FooBar")

	def test_detectOutputFormat_err3(self):
		res = Glossary.detectOutputFormat(
			filename="",
			formatName="",
			inputFilename="test",
		)
		self.assertIsNone(res)
		self.assertLogCritical("No filename nor format is given for output file")

	def test_detectOutputFormat_err4_1(self):
		res = Glossary.detectOutputFormat(
			filename="",
			formatName="BabylonBgl",
			inputFilename="test3.txt",
		)
		self.assertIsNone(res)
		self.assertLogCritical("plugin BabylonBgl does not support writing")

	def test_detectOutputFormat_err4_2(self):
		res = Glossary.detectOutputFormat(
			filename="test.bgl",
			formatName="",
			inputFilename="",
		)
		self.assertIsNone(res)
		self.assertLogCritical("plugin BabylonBgl does not support writing")

	def test_detectOutputFormat_err5(self):
		res = Glossary.detectOutputFormat(
			filename="test",
			formatName="",
			inputFilename="",
		)
		self.assertIsNone(res)
		self.assertLogCritical("Unable to detect output format!")

	def test_detectOutputFormat_err6(self):
		res = Glossary.detectOutputFormat(
			filename="test",
			formatName="Tabfile",
			inputFilename="",
			addExt=True,
		)
		self.assertEqual(res, ("test", "Tabfile", ""))
		self.assertLogError("inputFilename is empty")

	def test_init_infoBadType(self):
		try:
			Glossary(info=["a"])
		except Exception as e:
			self.assertEqual(str(type(e)), "<class 'TypeError'>")
			self.assertEqual(
				str(e),
				"Glossary: `info` has invalid type, dict or OrderedDict expected",
			)
		else:
			self.fail("did not raise an exception")

	def test_cleanup_removed(self):
		glos = Glossary()
		tmpFname = "test_cleanup_removed"
		entry = glos.newDataEntry(tmpFname, b"test")

		tmpFpath = entry.tmpPath
		self.assertTrue(bool(tmpFpath), msg="entry tmpPath is empty")
		if tmpFpath:
			self.assertTrue(
				isfile(tmpFpath),
				msg=f"tmp file does not exist: {tmpFpath}",
			)

		rmtree(appTmpDir)
		glos.cleanup()
		self.assertLogError(f"no such file or directory: {appTmpDir}")

	def test_lang_err_get_source(self):
		glos = Glossary()
		glos.setInfo("sourcelang", "test")
		self.assertEqual(glos.sourceLangName, "")
		self.assertLogError("unknown language 'test'")

	def test_lang_err_get_target(self):
		glos = Glossary()
		glos.setInfo("targetlang", "test")
		self.assertEqual(glos.targetLangName, "")
		self.assertLogError("unknown language 'test'")

	def test_lang_err_set_source(self):
		glos = Glossary()
		glos.sourceLangName = "foobar"
		self.assertLogError("unknown language 'foobar'")
		self.assertEqual(glos.sourceLangName, "")

	def test_lang_err_set_target(self):
		glos = Glossary()
		glos.targetLangName = "foobar"
		self.assertLogError("unknown language 'foobar'")
		self.assertEqual(glos.targetLangName, "")

	def test_lang_err_setObj_source(self):
		glos = Glossary()
		try:
			glos.sourceLang = "foobar"
		except TypeError as e:
			self.assertEqual(str(e), "invalid lang='foobar', must be a Lang object")
		else:
			self.fail("must raise a TypeError")

	def test_lang_err_setObj_target(self):
		glos = Glossary()
		try:
			glos.targetLang = "foobar"
		except TypeError as e:
			self.assertEqual(str(e), "invalid lang='foobar', must be a Lang object")
		else:
			self.fail("must raise a TypeError")

	def test_config_attr_set_twice(self):
		glos = Glossary()
		glos.config = {"lower": True}
		self.assertEqual(glos.getConfig("lower", False), True)
		glos.config = {"lower": False}
		self.assertLogError("glos.config is set more than once")
		self.assertEqual(glos.getConfig("lower", False), True)

	def test_iter_empty(self):
		glos = Glossary()
		self.assertEqual(list(glos), [])

	def test_convert_typeErr_1(self):
		glos = Glossary()
		try:
			glos.convert(
				inputFilename=1,
			)
		except TypeError as e:
			self.assertEqual(str(e), "inputFilename must be str")
		else:
			self.fail("must raise TypeError")

	def test_convert_typeErr_2(self):
		glos = Glossary()
		try:
			glos.convert(
				inputFilename="",
				outputFilename=1,
			)
		except TypeError as e:
			self.assertEqual(str(e), "outputFilename must be str")
		else:
			self.fail("must raise TypeError")

	def test_convert_typeErr_3(self):
		glos = Glossary()
		try:
			glos.convert(
				inputFilename="",
				outputFilename="",
				inputFormat=1,
			)
		except TypeError as e:
			self.assertEqual(str(e), "inputFormat must be str")
		else:
			self.fail("must raise TypeError")

	def test_convert_typeErr_4(self):
		glos = Glossary()
		try:
			glos.convert(
				inputFilename="",
				outputFilename="",
				inputFormat="",
				outputFormat=2,
			)
		except TypeError as e:
			self.assertEqual(str(e), "outputFormat must be str")
		else:
			self.fail("must raise TypeError")

	def test_read_typeErr_1(self):
		glos = Glossary()
		try:
			glos.read(
				filename=1,
			)
		except TypeError as e:
			self.assertEqual(str(e), "filename must be str")
		else:
			self.fail("must raise TypeError")

	def test_write_typeErr_1(self):
		glos = Glossary()
		try:
			glos.write(
				filename=1,
				format="",
			)
		except TypeError as e:
			self.assertEqual(str(e), "filename must be str")
		else:
			self.fail("must raise TypeError")

	def test_write_typeErr_2(self):
		glos = Glossary()
		try:
			glos.write(
				filename="",
				format=1,
			)
		except TypeError as e:
			self.assertEqual(str(e), "formatName must be str")
		else:
			self.fail("must raise TypeError")

	def test_convert_sameFilename(self):
		glos = Glossary()
		res = glos.convert(
			inputFilename="test4.txt",
			outputFilename="test4.txt",
		)
		self.assertIsNone(res)
		self.assertLogCritical("Input and output files are the same")

	def test_convert_dirExists(self):
		glos = Glossary()
		tempFilePath = self.newTempFilePath("test_convert_dirExists")
		with open(tempFilePath, mode="w", encoding="utf-8") as file:
			file.write("")
		res = glos.convert(
			inputFilename="test5.txt",
			outputFilename=self.tempDir,
			outputFormat="Stardict",
		)
		self.assertIsNone(res)
		self.assertLogCritical(
			f"Directory already exists and not empty: {relpath(self.tempDir)}",
		)

	def test_convert_fileNotFound(self):
		glos = Glossary()
		inputFilename = join(osRoot(), "abc", "def", "test6.txt")
		res = glos.convert(
			inputFilename=inputFilename,
			outputFilename="test2.txt",
		)
		self.assertIsNone(res)
		self.assertLogCritical(
			f"[Errno 2] No such file or directory: {inputFilename!r}",
		)
		self.assertLogCritical(f"Reading file {relpath(inputFilename)!r} failed.")

	def test_convert_unableDetectOutputFormat(self):
		glos = Glossary()
		res = glos.convert(
			inputFilename="test7.txt",
			outputFilename="test",
			outputFormat="",
		)
		self.assertIsNone(res)
		self.assertLogCritical("Unable to detect output format!")

	def test_convert_writeFileNotFound_txt(self):
		outputFilename = join(
			appTmpDir,
			"test",
			"7de8cf6f17bc4c9abb439e71adbec95d.txt",
		)
		glos = Glossary()
		res = glos.convert(
			inputFilename=self.downloadFile("100-en-fa.txt"),
			outputFilename=outputFilename,
		)
		self.assertIsNone(res)
		self.assertLogCritical(
			f"[Errno 2] No such file or directory: {outputFilename!r}",
		)
		self.assertLogCritical(f"Writing file {relpath(outputFilename)!r} failed.")

	def test_convert_writeFileNotFound_hdir(self):
		outputFilename = join(osRoot(), "test", "40e20107f5b04087bfc0ec0d61510017.hdir")
		glos = Glossary()
		res = glos.convert(
			inputFilename=self.downloadFile("100-en-fa.txt"),
			outputFilename=outputFilename,
		)
		self.assertIsNone(res)
		self.assertLogCritical(
			f"{osNoSuchFileOrDir} {outputFilename!r}",
		)
		self.assertLogCritical(f"Writing file {relpath(outputFilename)!r} failed.")

	def test_convert_invalidSortKeyName(self):
		glos = self.glos = Glossary()
		outputFilename = self.newTempFilePath("none.txt")
		res = glos.convert(
			inputFilename=self.downloadFile("100-en-fa.txt"),
			outputFilename=outputFilename,
			sort=True,
			sortKeyName="blah",
		)
		self.assertIsNone(res)
		self.assertLogCritical("invalid sortKeyName = 'blah'")

	def test_collectDefiFormat_direct(self):
		fname = "100-en-fa.txt"
		glos = self.glos = Glossary()
		glos.read(self.downloadFile(fname), direct=True)
		res = glos.collectDefiFormat(10)
		self.assertIsNone(res)
		self.assertLogError("collectDefiFormat: not supported in direct mode")

	def test_sortWords_invalidSortKeyName(self):
		glos = self.glos = Glossary()
		glos.sortWords(
			sortKeyName="blah",
		)
		self.assertLogCritical("invalid sortKeyName = 'blah'")

	# def test_collectDefiFormat_direct(self):
	# 	from pyglossary.glossary import Glossary as GlossaryLegacy
	# 	fname = "100-en-fa.txt"
	# 	glos = self.glos = GlossaryLegacy()
	# 	glos.read(self.downloadFile(fname), direct=True)
	# 	res = glos.collectDefiFormat(10)
	# 	self.assertIsNone(res)
	# 	self.assertLogError("collectDefiFormat: not supported in direct mode")


if __name__ == "__main__":
	unittest.main()



import os
import unittest

from glossary_v2_test import TestGlossaryBase


class TestGlossaryQuickDic6(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"100-en-de-v4.txt": "d420a669",
				"100-en-de-v4.txt.quickdic": "9d4ccc13",
				"100-en-de-v4.txt.quickdic.txt": "2dc4fc17",
				"100-en-fa.txt.quickdic": "2bd483df",
				"100-en-fa.txt.quickdic.txt": "50994fb5",
			},
		)

		os.environ["QUICKDIC_CREATION_TIME"] = "1730579400"

	def convert_txt_quickdic(self, fname, sha1sum, **convertArgs):
		self.convert(
			f"{fname}.txt",
			f"{fname}-2.quickdic",
			sha1sum=sha1sum,
			**convertArgs,
		)

	def convert_quickdic_txt(self, fname, fname2, **convertArgs):
		self.convert(
			f"{fname}.quickdic",
			f"{fname}-2.txt",
			compareText=f"{fname2}.txt",
			**convertArgs,
		)

	def test_convert_txt_quickdic_1(self):
		self.convert_txt_quickdic(
			"100-en-de-v4",
			"c8d9694624bace08e6e999db75c9156776f257c9",
		)

	def test_convert_quickdic_txt_1(self):
		self.convert_quickdic_txt(
			"100-en-de-v4.txt",
			"100-en-de-v4.txt.quickdic",
		)

	def test_convert_txt_quickdic_2(self):
		self.convert_txt_quickdic(
			"100-en-fa",
			"371ac30d5ddedffe0a1c54b8a050aef62e5b91a5",
		)

	def test_convert_quickdic_txt_2(self):
		self.convert_quickdic_txt(
			"100-en-fa.txt",
			"100-en-fa.txt.quickdic",
		)


if __name__ == "__main__":
	unittest.main()



import unittest
from os.path import join

from glossary_v2_test import TestGlossaryBase


class TestGlossaryBGL(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"Flavours_of_Malaysia.bgl": "46ef154b",
				"Flavours_of_Malaysia.txt_res/icon1.ico": "76a3b4c3",
				"Currency_In_Each_Country.bgl": "309f1b3f",
				"Solar_Physics_Glossary.bgl": "cc8f5ca1",
				"Farsi_Aviation_Dictionary.bgl": "efa7bee4",
			},
		)

	def convert_bgl_txt(
		self,
		fname,
		sha1sum=None,
		md5sum=None,
		resFiles=None,
		**convertArgs,
	):
		if resFiles is None:
			resFiles = {}
		resFilesPath = {
			resName: self.newTempFilePath(join(f"{fname}-2.txt_res", resName))
			for resName in resFiles
		}

		self.convert(
			f"{fname}.bgl",
			f"{fname}-2.txt",
			sha1sum=sha1sum,
			md5sum=md5sum,
			**convertArgs,
		)

		for resName in resFiles:
			resPathActual = resFilesPath[resName]
			resPathExpected = self.downloadFile(f"{fname}.txt_res/{resName}")
			self.compareBinaryFiles(resPathActual, resPathExpected)

	def test_convert_bgl_txt_1(self):
		self.convert_bgl_txt(
			"Flavours_of_Malaysia",
			sha1sum="2b1fae135df2aaaeac23fb1dde497a4b6a22fd95",
			resFiles=["icon1.ico"],
		)

	def test_convert_bgl_txt_2(self):
		self.convert_bgl_txt(
			"Currency_In_Each_Country",
			sha1sum="731147c72092d813dfe1ab35d420477478832443",
		)

	def test_convert_bgl_txt_3(self):
		self.convert_bgl_txt(
			"Solar_Physics_Glossary",
			sha1sum="f30b392c748c4c5bfa52bf7f9945c574617ff74a",
		)

	def test_convert_bgl_txt_4(self):
		self.convert_bgl_txt(
			"Farsi_Aviation_Dictionary",
			sha1sum="34729e2542085c6026090e9e3f49d10291393113",
			readOptions={
				"process_html_in_key": True,
			},
		)


if __name__ == "__main__":
	unittest.main()



# -*- coding: utf-8 -*-
# mypy: ignore-errors
#
# Copyright © 2025 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
#
# This program is a free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# You can get a copy of GNU General Public License along this program
# But you can always get it from http://www.gnu.org/licenses/gpl.txt
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

from __future__ import annotations

import ast
import logging
from typing import TYPE_CHECKING, Any, Protocol

from gi.repository import Gtk as gtk

from pyglossary.core import pip
from pyglossary.glossary_v2 import Glossary
from pyglossary.option import Option
from pyglossary.text_utils import escapeNRB, unescapeNRB
from pyglossary.ui.dependency import checkDepends

from .utils import (
	HBox,
	pack,
	showInfo,
)

if TYPE_CHECKING:
	from collections.abc import Callable

	from pyglossary.plugin_prop import PluginProp

__all__ = ["FormatOptionsDialog", "InputFormatBox", "OutputFormatBox"]


_ = str

pluginByDesc = {plugin.description: plugin for plugin in Glossary.plugins.values()}
readDesc = [plugin.description for plugin in Glossary.plugins.values() if plugin.canRead]
writeDesc = [
	plugin.description for plugin in Glossary.plugins.values() if plugin.canWrite
]

log = logging.getLogger("pyglossary")


# TODO: Dialog is deprecated since version 4.10, Use Window instead
class FormatDialog(gtk.Dialog):
	def __init__(
		self,
		descList: list[str],
		parent: gtk.Widget | None = None,
		**kwargs: Any,
	) -> None:
		gtk.Dialog.__init__(self, transient_for=parent, **kwargs)
		self.set_default_size(400, 400)
		self.vbox = self.get_content_area()
		##
		self.descList = descList
		self.items = descList
		self.activeDesc = ""
		##
		self.connect("response", lambda _w, _e: self.hide())
		self.add_action_widget(
			gtk.Button(
				label="_Cancel",
				use_underline=True,
				# icon_name="gtk-cancel",
			),
			gtk.ResponseType.CANCEL,
		)
		self.add_action_widget(
			gtk.Button(
				label="_OK",
				use_underline=True,
				# icon_name="gtk-ok",
			),
			gtk.ResponseType.OK,
		)
		###
		treev = gtk.TreeView()
		treeModel = gtk.ListStore(str)
		treev.set_headers_visible(False)
		treev.set_model(treeModel)
		treev.connect("row-activated", self.rowActivated)
		# treev.connect("response", self.onResponse)
		###
		self.treev = treev
		#############
		cell = gtk.CellRendererText(editable=False)
		col = gtk.TreeViewColumn(
			title="Descriptin",
			cell_renderer=cell,
			text=0,
		)
		col.set_property("expand", True)
		col.set_resizable(True)
		treev.append_column(col)
		self.descCol = col
		############
		hbox = HBox(spacing=15)
		hbox.get_style_context().add_class("margin_05")
		pack(hbox, gtk.Label(label="Search:"))
		entry = self.entry = gtk.Entry()
		pack(hbox, entry, 1, 1)
		pack(self.vbox, hbox)
		###
		entry.connect("changed", self.onEntryChange)
		############
		self.swin = swin = gtk.ScrolledWindow()
		swin.set_child(treev)
		swin.set_policy(gtk.PolicyType.NEVER, gtk.PolicyType.AUTOMATIC)
		pack(self.vbox, swin, 1, 1)
		self.vbox.show()
		##
		treev.set_can_focus(True)  # no need, just to be safe
		# treev.set_can_default(True)
		treev.set_receives_default(True)
		# print("can_focus:", treev.get_can_focus())
		# print("can_default:", treev.get_can_default())
		# print("receives_default:", treev.get_receives_default())
		####
		self.updateTree()
		self.connect("realize", self.onRealize)

	def onRealize(self, _widget: gtk.Widget = None) -> None:
		if self.activeDesc:
			self.treev.grab_focus()
		else:
			self.entry.grab_focus()

	def onEntryChange(self, entry: gtk.Entry) -> None:
		text = entry.get_text().strip()
		if not text:
			self.items = self.descList
			self.updateTree()
			return

		text = text.lower()
		descList = self.descList

		items1 = []
		items2 = []
		for desc in descList:
			if desc.lower().startswith(text):
				items1.append(desc)
			elif text in desc.lower():
				items2.append(desc)

		self.items = items1 + items2
		self.updateTree()

	def setCursor(self, desc: str) -> None:
		model = self.treev.get_model()
		iter_ = model.iter_children(None)
		while iter_ is not None:
			if model.get_value(iter_, 0) == desc:
				path = model.get_path(iter_)
				self.treev.set_cursor(path, self.descCol, False)
				self.treev.scroll_to_cell(path)
				return
			iter_ = model.iter_next(iter_)

	def updateTree(self) -> None:
		model = self.treev.get_model()
		model.clear()
		for desc in self.items:
			model.append([desc])

		if self.activeDesc:
			self.setCursor(self.activeDesc)

	def getActive(self) -> PluginProp | None:
		iter_ = self.treev.get_selection().get_selected()[1]
		if iter_ is None:
			return None
		model = self.treev.get_model()
		desc = model.get_value(iter_, 0)
		return pluginByDesc[desc]

	def setActive(self, plugin: PluginProp) -> None:
		if plugin is None:
			self.activeDesc = ""
			return
		desc = plugin.description
		self.activeDesc = desc
		self.setCursor(desc)

	def rowActivated(
		self,
		treev: gtk.TreeView,
		path: gtk.GtkTreePath,
		_col: object,
	) -> None:
		model = treev.get_model()
		iter_ = model.get_iter(path)
		desc = model.get_value(iter_, 0)
		self.activeDesc = desc
		self.response(gtk.ResponseType.OK)

	# def onResponse


class FormatButton(gtk.Button):
	noneLabel = "[Select Format]"
	dialogTitle = "Select Format"

	def __init__(
		self,
		descList: list[str],
		onChanged: Callable,
		parent: gtk.Widget | None = None,
	) -> None:
		gtk.Button.__init__(self)
		self.set_label(self.noneLabel)
		###
		self.descList = descList
		self.onChanged = onChanged
		self._parent = parent
		self.activePlugin = None
		###
		self.connect("clicked", self.onClick)

	def onDialogResponse(self, dialog: gtk.Dialog, response_id: int) -> None:
		print(f"onDialogResponse: {dialog}, {response_id}")
		if response_id != gtk.ResponseType.OK:
			return

		plugin = dialog.getActive()
		self.activePlugin = plugin
		if plugin:
			self.set_label(plugin.description)
		else:
			self.set_label(self.noneLabel)
		self.onChanged()

	def onClick(self, _button: gtk.Widget = None) -> None:
		dialog = FormatDialog(
			descList=self.descList,
			parent=self._parent,
			title=self.dialogTitle,
		)
		dialog.setActive(self.activePlugin)
		dialog.connect("response", self.onDialogResponse)
		dialog.present()

	def getActive(self) -> str:
		if self.activePlugin is None:
			return ""
		return self.activePlugin.name

	def setActive(self, formatName: str) -> None:
		plugin = Glossary.plugins[formatName]
		self.activePlugin = plugin
		self.set_label(plugin.description)
		self.onChanged()


class OptionTkType(Protocol):
	def __init__(self, opt: Option) -> None: ...
	@property
	def value(self) -> Any: ...
	@value.setter
	def value(self, x: Any) -> None: ...
	@property
	def widget(self) -> gtk.Widget: ...


class BoolOptionGtk:
	def __init__(self, opt: Option) -> None:
		hbox = HBox()
		cb = gtk.CheckButton(
			label=f"{opt.displayName} ({opt.comment})",
		)
		pack(hbox, cb)
		self._hbox = hbox
		self._cb = cb

	@property
	def value(self) -> Any:
		return self._cb.get_active()

	@value.setter
	def value(self, x: Any) -> None:
		self._cb.set_active(bool(x))

	@property
	def widget(self) -> gtk.Widget:
		return self._hbox


class IntOptionGtk:
	def __init__(self, opt: Option) -> None:
		hbox = HBox()
		pack(
			hbox,
			gtk.Label(label=f"{opt.displayName}: "),
		)
		minim = opt.minim
		if minim is None:
			minim = 0
		maxim = opt.maxim
		if maxim is None:
			maxim = 1_000_000_000
		spin = gtk.SpinButton()
		spin.set_digits(0)
		spin.set_range(minim, maxim)
		spin.set_increments(1, 10)
		spin.set_width_chars(max(len(str(minim)), len(str(maxim))))
		pack(hbox, spin)
		self._hbox = hbox
		self._spin = spin

	@property
	def value(self) -> Any:
		return int(self._spin.get_value())

	@value.setter
	def value(self, x: Any) -> None:
		self._spin.set_value(int(x))

	@property
	def widget(self) -> gtk.Widget:
		return self._hbox


class StrOptionGtk:
	def __init__(self, opt: Option) -> None:
		self.opt = opt
		hbox = HBox()
		pack(
			hbox,
			gtk.Label(label=f"{opt.displayName} ({opt.comment}): "),
		)
		self._hbox = hbox
		if opt.customValue:
			combo = gtk.ComboBoxText.new_with_entry()
		else:
			combo = gtk.ComboBoxText.new()
		self._combo = combo
		pack(hbox, combo)
		if opt.values:
			for value in opt.values:
				combo.append_text(value)
			self._width = max(len(x) for x in opt.values)
		else:
			self._width = 10
		combo.get_child().set_width_chars(self._width)

	@property
	def value(self) -> Any:
		return self._combo.get_active_text()

	@value.setter
	def value(self, x: Any) -> None:
		st = str(x)
		values = self.opt.values or []
		try:
			index = values.index(st)
		except ValueError:
			self._combo.append_text(st)
			index = len(self._combo.get_model()) - 1
		self._combo.set_active(index)
		if len(st) > self._width:
			self._width = len(st)
			self._combo.get_child().set_width_chars(len(st))

	@property
	def widget(self) -> gtk.Widget:
		return self._hbox


# should I use str input (Entry) to allow values like 1m and 1g?
class FileSizeOptionGtk(IntOptionGtk):
	pass


class HtmlColorOptionGtk(StrOptionGtk):
	pass


class MultiLineStrOptionGtk:
	def __init__(self, opt: Option) -> None:
		hbox = HBox()
		pack(
			hbox,
			gtk.Label(label=f"{opt.displayName} ({opt.comment}): "),
		)
		tview = gtk.TextView()
		frame = gtk.Frame()
		frame.set_child(tview)
		pack(hbox, frame, expand=True)
		self._hbox = hbox
		self._buf = tview.get_buffer()

	@property
	def value(self) -> Any:
		return self._buf.get_text(
			start=self._buf.get_start_iter(),
			end=self._buf.get_end_iter(),
			include_hidden_chars=True,
		)

	@value.setter
	def value(self, x: Any) -> None:
		self._buf.set_text(str(x))

	@property
	def widget(self) -> gtk.Widget:
		return self._hbox


class NewlineOptionGtk:
	def __init__(self, opt: Option) -> None:
		self.opt = opt
		hbox = HBox()
		pack(
			hbox,
			gtk.Label(
				label=f"{opt.displayName} ({opt.comment}, escaped \\n\\r): ",
			),
		)
		self._hbox = hbox
		combo = gtk.ComboBoxText.new_with_entry()
		for value in opt.values:
			combo.append_text(escapeNRB(value))
		self._combo = combo
		pack(hbox, combo)
		combo.get_child().set_width_chars(5)

	@property
	def value(self) -> Any:
		return unescapeNRB(self._combo.get_active_text())

	@value.setter
	def value(self, x: Any) -> None:
		st = escapeNRB(str(x))
		values = self.opt.values or []
		try:
			index = values.index(st)
		except ValueError:
			self._combo.append_text(st)
			index = len(self._combo.get_model()) - 1
		self._combo.set_active(index)

	@property
	def widget(self) -> gtk.Widget:
		return self._hbox


class LiteralEvalOptionGtk(MultiLineStrOptionGtk):
	@property
	def value(self) -> Any:
		return ast.literal_eval(self._buf.get_text())

	@value.setter
	def value(self, x: Any) -> None:
		self._buf.set_text(repr(x))


class ListOptionGtk(LiteralEvalOptionGtk):
	typeHint = "Python list"


class DictOptionGtk(LiteralEvalOptionGtk):
	typeHint = "Python dict"


optionClassByName: dict[str, OptionTkType] = {
	"BoolOption": BoolOptionGtk,
	"IntOption": IntOptionGtk,
	"StrOption": StrOptionGtk,
	"EncodingOption": StrOptionGtk,
	"FileSizeOption": FileSizeOptionGtk,
	"UnicodeErrorsOption": StrOptionGtk,
	"HtmlColorOption": HtmlColorOptionGtk,
	"NewlineOption": NewlineOptionGtk,
	"ListOption": ListOptionGtk,
	"DictOption": DictOptionGtk,
	# "FloatOption": FloatOptionGtk,  # not used so far!
}


class FormatOptionsDialog(gtk.Dialog):
	commentLen = 60
	kindFormatsOptions = {
		"r": Glossary.formatsReadOptions,
		"w": Glossary.formatsWriteOptions,
	}

	def __init__(
		self,
		app: gtk.Application,
		formatName: str,
		kind: str,  # "r" or "w"
		values: dict[str, Any],
		**kwargs: Any,
	) -> None:
		self.app = app
		gtk.Dialog.__init__(self, **kwargs)
		self.vbox = self.get_content_area()
		##
		optionsProp = Glossary.plugins[formatName].optionsProp
		self.optionsProp = optionsProp
		self.formatName = formatName
		##
		self.connect("response", lambda _w, _e: self.hide())
		self.add_action_widget(
			gtk.Button(
				label="_Cancel",
				use_underline=True,
				# icon_name="gtk-cancel",
			),
			gtk.ResponseType.CANCEL,
		)
		self.add_action_widget(
			gtk.Button(
				label="_OK",
				use_underline=True,
				# icon_name="gtk-ok",
			),
			gtk.ResponseType.OK,
		)
		###
		self.options = self.kindFormatsOptions[kind][formatName]
		self.values = values
		self.widgets: dict[str, OptionTkType] = {}
		self.createOptionsList()
		############
		self.vbox.show()

	def createOptionsList(self) -> None:
		values = self.values
		for optName, default in self.options.items():
			prop = self.optionsProp[optName]
			comment = prop.longComment
			if len(comment) > self.commentLen:
				comment = comment[: self.commentLen] + "..."
			widgetClass = optionClassByName.get(prop.__class__.__name__)
			if widgetClass is None:
				log.warning(f"No widget class for option class {prop.__class__}")
				continue
			w = widgetClass(prop)
			pack(self.vbox, w.widget)
			w.value = values.get(optName, default)
			self.widgets[optName] = w

	def getOptionsValues(self) -> dict[str, Any]:
		optionsValues: dict[str, Any] = {}
		for optName, widget in self.widgets.items():
			optionsValues[optName] = widget.value
		return optionsValues


class FormatBox(gtk.Box):
	def __init__(
		self,
		app: gtk.Application,
		descList: list[str],
		parent: gtk.Widget | None = None,
		labelSizeGroup: gtk.SizeGroup = None,
	) -> None:
		gtk.Box.__init__(self, orientation=gtk.Orientation.HORIZONTAL, spacing=3)

		self.app = app
		self._parent = parent
		self.button = FormatButton(
			descList,
			parent=parent,
			onChanged=self.onChanged,
		)

		self.dependsButton = gtk.Button(label="Install dependencies")
		self.dependsButton.pkgNames = []
		self.dependsButton.connect("clicked", self.dependsButtonClicked)

		label = gtk.Label(label=_("Input Format:"), xalign=0)
		pack(self, label)
		pack(self, self.button)
		pack(self, gtk.Label(), 1, 1)
		pack(self, self.dependsButton)

		self.show()
		self.dependsButton.hide()

		if labelSizeGroup:
			labelSizeGroup.add_widget(label)

	def getActive(self) -> str:
		return self.button.getActive()

	def setActive(self, active: str) -> None:
		self.button.setActive(active)

	def kind(self) -> str:
		"""Return 'r' or 'w'."""
		raise NotImplementedError

	def dependsButtonClicked(self, button: gtk.Button) -> None:
		formatName = self.getActive()
		pkgNames = button.pkgNames
		if not pkgNames:
			print("All dependencies are stattisfied for " + formatName)
			return
		pkgNamesStr = " ".join(pkgNames)
		msg = f"Run the following command:\n{pip} install {pkgNamesStr}"
		showInfo(
			msg,
			title="Dependencies for " + formatName,
			selectable=True,
			parent=self._parent,
		)
		self.onChanged(self)

	def onChanged(self, _obj: gtk.Widget = None) -> None:
		name = self.getActive()
		if not name:
			return

		kind = self.kind()
		plugin = Glossary.plugins[name]
		if kind == "r":
			depends = plugin.readDepends
		elif kind == "w":
			depends = plugin.writeDepends
		else:
			raise RuntimeError(f"invalid {kind=}")
		uninstalled = checkDepends(depends)

		self.dependsButton.pkgNames = uninstalled
		self.dependsButton.set_visible(bool(uninstalled))


class InputFormatBox(FormatBox):
	dialogTitle = "Select Input Format"

	def __init__(self, app: gtk.Application, **kwargs: Any) -> None:
		FormatBox.__init__(self, app, readDesc, **kwargs)

	def kind(self) -> str:
		"""Return 'r' or 'w'."""
		return "r"


class OutputFormatBox(FormatBox):
	dialogTitle = "Select Output Format"

	def __init__(self, app: gtk.Application, **kwargs: Any) -> None:
		FormatBox.__init__(self, app, writeDesc, **kwargs)

	def kind(self) -> str:
		"""Return 'r' or 'w'."""
		return "w"



# -*- coding: utf-8 -*-

from __future__ import annotations

import typing
import warnings
from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
	from collections.abc import Callable

__all__ = ["BaseEntry", "MultiStr"]

MultiStr: typing.TypeAlias = "str | list[str]"


class BaseEntry(ABC):  # noqa: PLR0904
	__slots__: list[str] = [
		"_term",
	]

	def __init__(self) -> None:
		self._term: str | list[str]

	@classmethod
	@abstractmethod
	def isData(cls) -> bool:
		raise NotImplementedError

	@abstractmethod
	def getFileName(self) -> str:
		raise NotImplementedError

	@property
	@abstractmethod
	def s_term(self) -> str:
		raise NotImplementedError

	@property
	@abstractmethod
	def l_term(self) -> list[str]:
		raise NotImplementedError

	@property
	@abstractmethod
	def lb_term(self) -> list[bytes]:
		raise NotImplementedError

	@property
	@abstractmethod
	def defi(self) -> str:
		raise NotImplementedError

	@property
	@abstractmethod
	def defiFormat(self) -> str:
		# TODO: type: Literal["m", "h", "x", "b"]
		raise NotImplementedError

	@defiFormat.setter
	@abstractmethod
	def defiFormat(self, defiFormat: str) -> None:
		# TODO: type: Literal["m", "h", "x", "b"]
		raise NotImplementedError

	@abstractmethod
	def detectDefiFormat(self, default: str = "") -> str:
		raise NotImplementedError

	@abstractmethod
	def addAlt(self, alt: str) -> None:
		raise NotImplementedError

	@abstractmethod
	def editFuncTerm(self, func: Callable[[str], str]) -> None:
		raise NotImplementedError

	@abstractmethod
	def editFuncDefi(self, func: Callable[[str], str]) -> None:
		raise NotImplementedError

	@abstractmethod
	def strip(self) -> None:
		raise NotImplementedError

	@abstractmethod
	def replaceInWord(self, source: str, target: str) -> None:
		raise NotImplementedError

	@abstractmethod
	def replaceInDefi(self, source: str, target: str) -> None:
		raise NotImplementedError

	@abstractmethod
	def replace(self, source: str, target: str) -> None:
		raise NotImplementedError

	@abstractmethod
	def byteProgress(self) -> tuple[int, int] | None:
		raise NotImplementedError

	@abstractmethod
	def removeEmptyAndDuplicateAltTerms(self) -> None:
		raise NotImplementedError

	@abstractmethod
	def stripFullHtml(self) -> str | None:
		raise NotImplementedError

	# ________________________________________________________

	@property
	def data(self) -> bytes:
		raise NotImplementedError

	def size(self) -> int:
		raise NotImplementedError

	def save(self, directory: str) -> str:
		raise NotImplementedError

	@property
	def tmpPath(self) -> str | None:
		raise NotImplementedError

	@property
	def b_term(self) -> bytes:
		"""Returns bytes of main term and all the alternate terms separated by b"|"."""
		return self.s_term.encode("utf-8")

	@property
	def b_defi(self) -> bytes:
		"""Returns definition in bytes."""
		return self.defi.encode("utf-8")

	# ________________________________________________________

	@property
	def b_word(self) -> bytes:
		warnings.warn(
			"entry.b_word is deprecated, use entry.b_term",
			category=DeprecationWarning,
			stacklevel=3,
		)
		return self.s_term.encode("utf-8")

	@property
	def s_word(self) -> str:
		warnings.warn(
			"entry.s_word is deprecated, use entry.s_term",
			category=DeprecationWarning,
			stacklevel=3,
		)
		return self.s_term

	@property
	def l_word(self) -> list[str]:
		warnings.warn(
			"entry.l_word is deprecated, use entry.l_term",
			category=DeprecationWarning,
			stacklevel=3,
		)
		return self.l_term

	@property
	def lb_word(self) -> list[bytes]:
		warnings.warn(
			"entry.lb_word is deprecated, use entry.lb_term",
			category=DeprecationWarning,
			stacklevel=3,
		)
		return self.lb_term

	def editFuncWord(self, func: Callable[[str], str]) -> None:
		warnings.warn(
			"entry.editFuncWord is deprecated, use entry.editFuncTerm",
			category=DeprecationWarning,
			stacklevel=3,
		)
		self.editFuncTerm(func)



import unittest

from glossary_v2_test import TestGlossaryBase


class TestGlossaryDictOrg(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"100-en-fa.txt.dict": "02abe5dc",
				"100-en-fa.txt.index": "b10efcb4",
				"100-en-fa.txt.index.txt": "6c9d527c",
			},
		)

	def convert_txt_dict_org(self, fname, fname2, **convertArgs):
		self.convert(
			f"{fname}.txt",
			f"{fname}-2.index",
			compareText=f"{fname2}.index",
			**convertArgs,
		)

	def convert_dict_org_txt(self, fname, fname2, **convertArgs):
		self.downloadFile(f"{fname}.dict")
		self.convert(
			f"{fname}.index",
			f"{fname}-2.txt",
			compareText=f"{fname2}.txt",
			**convertArgs,
		)

	def test_convert_txt_dict_org_1(self):
		self.convert_txt_dict_org(
			"100-en-fa",
			"100-en-fa.txt",
			writeOptions={"install": False},
		)

	def test_convert_dict_org_txt_1(self):
		self.convert_dict_org_txt(
			"100-en-fa.txt",
			"100-en-fa.txt.index",
			infoOverride={"input_file_size": None},
		)


if __name__ == "__main__":
	unittest.main()



from __future__ import annotations

import sys
import unittest
from os.path import abspath, dirname

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)

from pyglossary.entry import Entry


class TestEntryBasic(unittest.TestCase):
	def test_exc_deprecated_0(self):
		Entry(word="word", defi="defi")

	def test_exc_1(self):
		try:
			Entry(b"word", "defi")
		except TypeError as e:
			self.assertEqual(str(e), "invalid term type <class 'bytes'>")
		else:
			self.fail("must raise TypeError")

	def test_exc_2(self):
		Entry(("word",), "defi")

	def test_exc_3(self):
		try:
			Entry("word", b"defi")
		except TypeError as e:
			self.assertEqual(str(e), "invalid defi type <class 'bytes'>")
		else:
			self.fail("must raise TypeError")

	def test_exc_4(self):
		try:
			Entry("word", ("defi",))
		except TypeError as e:
			self.assertEqual(str(e), "invalid defi type <class 'tuple'>")
		else:
			self.fail("must raise TypeError")

	def test_exc_5(self):
		try:
			Entry("word", "defi", "b")
		except ValueError as e:
			self.assertEqual(str(e), "invalid defiFormat 'b'")
		else:
			self.fail("must raise ValueError")

	def test_1(self):
		entry = Entry("test1", "something")
		self.assertEqual(entry.l_term, ["test1"])
		self.assertEqual(entry.defi, "something")

	def test_2(self):
		entry = Entry(["test1"], "something")
		self.assertEqual(entry.l_term, ["test1"])
		self.assertEqual(entry.defi, "something")

	def test_3(self):
		entry = Entry("test1", ["something"])
		self.assertEqual(entry.l_term, ["test1"])
		self.assertEqual(entry.defi, "something")

	def test_repr_1(self):
		entry = Entry("test1", "something")
		self.assertEqual(
			repr(entry),
			"Entry('test1', 'something', defiFormat='m')",
		)

	def test_repr_2(self):
		entry = Entry("test1", "something", defiFormat="h")
		self.assertEqual(
			repr(entry),
			"Entry('test1', 'something', defiFormat='h')",
		)

	def test_defiFormat_1(self):
		entry = Entry("test1", "something")
		self.assertEqual(entry.defiFormat, "m")

	def test_defiFormat_2(self):
		entry = Entry("test1", "something", defiFormat="h")
		self.assertEqual(entry.defiFormat, "h")

	def test_defiFormat_3(self):
		entry = Entry("test1", "something", defiFormat="h")
		entry.defiFormat = "x"
		self.assertEqual(entry.defiFormat, "x")

	def test_addAlt_1(self):
		entry = Entry("test1", "something")
		self.assertEqual(entry.l_term, ["test1"])
		entry.addAlt("test 1")
		self.assertEqual(entry.l_term, ["test1", "test 1"])


class TestEntryDetectDefiFormat(unittest.TestCase):
	def test_1(self):
		entry = Entry("test1", "something")
		entry.detectDefiFormat()
		self.assertEqual(entry.defiFormat, "m")

	def test_2(self):
		entry = Entry("test1", "something", defiFormat="h")
		entry.detectDefiFormat()
		self.assertEqual(entry.defiFormat, "h")

	def test_3(self):
		entry = Entry("test1", "something", defiFormat="x")
		entry.detectDefiFormat()
		self.assertEqual(entry.defiFormat, "x")

	def test_4(self):
		entry = Entry("test1", "<b>something</b>")
		entry.detectDefiFormat()
		self.assertEqual(entry.defiFormat, "h")

	def test_5(self):
		entry = Entry("test1", "<k>title</k>something")
		entry.detectDefiFormat()
		self.assertEqual(entry.defiFormat, "x")


class TestEntryStripFullHtml(unittest.TestCase):
	def __init__(self, *args, **kwargs):
		unittest.TestCase.__init__(self, *args, **kwargs)

	def setUp(self):
		pass

	def tearDown(self):
		pass

	def case(
		self,
		word: str,
		origDefi: str,
		fixedDefi: str,
		error: str | None = None,
	):
		entry = Entry(word, origDefi)
		actualError = entry.stripFullHtml()
		self.assertEqual(entry.defi, fixedDefi)
		self.assertEqual(actualError, error)

	def test_1(self):
		self.case(
			word="test1",
			origDefi="plain text",
			fixedDefi="plain text",
			error=None,
		)

	def test_2(self):
		self.case(
			word="test2",
			origDefi="<p>simple <i>html</i> text</p>",
			fixedDefi="<p>simple <i>html</i> text</p>",
			error=None,
		)

	def test_3(self):
		self.case(
			word="test3",
			origDefi=(
				"<!DOCTYPE html><html><head></head><body>simple <i>html</i></body></html>"
			),
			fixedDefi="simple <i>html</i>",
			error=None,
		)

	def test_4(self):
		self.case(
			word="test4",
			origDefi="<html><head></head><body>simple <i>html</i></body></html>",
			fixedDefi="simple <i>html</i>",
			error=None,
		)

	def test_5(self):
		self.case(
			word="test5",
			origDefi="<!DOCTYPE html><html><head></head>simple <i>html</i></html>",
			fixedDefi="<!DOCTYPE html><html><head></head>simple <i>html</i></html>",
			error="<body not found",
		)

	def test_6(self):
		self.case(
			word="test6",
			origDefi="<html><head></head>no <body",
			fixedDefi="<html><head></head>no <body",
			error="'>' after <body not found",
		)

	def test_7(self):
		self.case(
			word="test7",
			origDefi="<html><head></head><body>",
			fixedDefi="<html><head></head><body>",
			error="</body close not found",
		)


if __name__ == "__main__":
	unittest.main()



import unittest

from glossary_v2_test import TestGlossaryBase


class TestGlossaryLingoesLDF(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"004-bar.ldf": "b1aa776d",
				"100-en-fa.ldf": "503d1a9b",
			},
		)

	def convert_txt_ldf(self, fname, fname2, **convertArgs):
		self.convert(
			f"{fname}.txt",
			f"{fname}-2.ldf",
			compareText=f"{fname2}.ldf",
			**convertArgs,
		)

	def convert_ldf_txt(self, fname, fname2, **convertArgs):
		self.convert(
			f"{fname}.ldf",
			f"{fname}-2.txt",
			compareText=f"{fname2}.txt",
			**convertArgs,
		)

	def test_convert_txt_ldf_1(self):
		self.convert_txt_ldf(
			"004-bar",
			"004-bar",
		)

	def test_convert_txt_ldf_2(self):
		self.convert_txt_ldf(
			"100-en-fa",
			"100-en-fa",
		)

	def test_convert_ldf_txt_1(self):
		self.convert_ldf_txt(
			"004-bar",
			"004-bar",
			infoOverride={
				"name": None,
				"input_file_size": None,
			},
		)


if __name__ == "__main__":
	unittest.main()



import os
import shutil
import time
from copy import copy
from zipfile import ZipFile, ZipInfo

try:
	from zipfile import _MASK_COMPRESS_OPTION_1  # type: ignore[attr-defined]
except ImportError:
	_MASK_COMPRESS_OPTION_1 = 0x02

__all__ = ["ReproducibleZipFile"]
__version__ = "0.3.1"


def date_time() -> time.struct_time | tuple[int, int, int, int, int, int]:
	"""
	Returns date_time value used to force overwrite on all ZipInfo objects. Defaults to
	1980-01-01 00:00:00. You can set this with the environment variable
	SOURCE_DATE_EPOCH as an integer value representing seconds since Epoch.
	"""
	source_date_epoch = os.environ.get("SOURCE_DATE_EPOCH", None)
	if source_date_epoch is not None:
		return time.gmtime(int(source_date_epoch))
	return (1980, 1, 1, 0, 0, 0)


def file_mode() -> int:
	"""
	Returns the file permissions mode value used to force overwrite
		on all ZipInfo objects.
	Defaults to 0o644 (rw-r--r--). You can set this with the environment variable
	REPRO_ZIPFILE_FILE_MODE. It should be in the Unix standard three-digit
		octal representation (e.g., '644').
	"""
	file_mode_env = os.environ.get("REPRO_ZIPFILE_FILE_MODE", None)
	if file_mode_env is not None:
		return int(file_mode_env, 8)
	return 0o644


def dir_mode() -> int:
	"""
	Returns the directory permissions mode value used to force overwrite
		on all ZipInfo objects.
	Defaults to 0o755 (rwxr-xr-x). You can set this with the environment variable
	REPRO_ZIPFILE_DIR_MODE. It should be in the Unix standard three-digit
	octal representation (e.g., '755').
	"""
	dir_mode_env = os.environ.get("REPRO_ZIPFILE_DIR_MODE", None)
	if dir_mode_env is not None:
		return int(dir_mode_env, 8)
	return 0o755


class ReproducibleZipFile(ZipFile):
	"""
		Open a ZIP file, where file can be a path to a file (a string),
		a file-like object or a path-like object.

		This is a replacement for the Python standard library zipfile.ZipFile that
		overwrites file-modified timestamps and file/directory permissions modes in
		write mode in order to create a reproducible ZIP archive. Other than overwriting
	these values, it works the same way as zipfile.ZipFile. For documentation on use,
	see the Python documentation for zipfile:
		https://docs.python.org/3/library/zipfile.html
	"""

	# Following method modified from Python 3.11
	# https://github.com/python/cpython/blob/202efe1a3bcd499f3bf17bd953c6d36d47747e78/Lib/zipfile.py#L1763-L1794
	# Copyright Python Software Foundation, licensed under PSF License Version 2
	# See LICENSE file for full license agreement and notice of copyright
	def write(
		self,
		filename: str,
		arcname: str | None = None,
		compress_type: str | None = None,
		compresslevel: int | None = None,
	) -> None:
		"""Put the bytes from filename into the archive under the name arcname."""
		if not self.fp:
			raise ValueError("Attempt to write to ZIP archive that was already closed")
		if self._writing:
			raise ValueError(
				"Can't write to ZIP archive while an open writing handle exists"
			)

		zinfo = ZipInfo.from_file(
			filename, arcname, strict_timestamps=self._strict_timestamps
		)

		# repro-zipfile ADDED ##
		# Overwrite date_time and extrnal_attr (permissions mode)
		zinfo = copy(zinfo)
		zinfo.date_time = date_time()
		if zinfo.is_dir():
			zinfo.external_attr = (0o40000 | dir_mode()) << 16
			zinfo.external_attr |= 0x10  # MS-DOS directory flag
		else:
			zinfo.external_attr = file_mode() << 16
		#########################

		if zinfo.is_dir():
			zinfo.compress_size = 0
			zinfo.CRC = 0
			self.mkdir(zinfo)
		else:
			if compress_type is not None:
				zinfo.compress_type = compress_type
			else:
				zinfo.compress_type = self.compression

			if compresslevel is not None:
				zinfo._compresslevel = compresslevel
			else:
				zinfo._compresslevel = self.compresslevel

			with open(filename, "rb") as src, self.open(zinfo, "w") as dest:
				shutil.copyfileobj(src, dest, 1024 * 8)

	# Following method modified from Python 3.11
	# https://github.com/python/cpython/blob/202efe1a3bcd499f3bf17bd953c6d36d47747e78/Lib/zipfile.py#L1796-L1835
	# Copyright Python Software Foundation, licensed under PSF License Version 2
	# See LICENSE file for full license agreement and notice of copyright
	def writestr(
		self,
		zinfo_or_arcname: ZipInfo | str,
		data: str | bytes,
		compress_type: str | None = None,
		compresslevel: int | None = None,
	) -> None:
		"""
		Write a file into the archive.  The contents is 'data', which may be either
		a 'str' or a 'bytes' instance; if it is a 'str', it is encoded as UTF-8 first.
		'zinfo_or_arcname' is either a ZipInfo instance or the name of the file in
		the archive.
		"""
		if isinstance(data, str):
			data = data.encode("utf-8")
		if not isinstance(zinfo_or_arcname, ZipInfo):
			zinfo = ZipInfo(
				filename=zinfo_or_arcname, date_time=time.localtime(time.time())[:6]
			)
			zinfo.compress_type = self.compression
			zinfo._compresslevel = self.compresslevel
			if zinfo.filename.endswith("/"):
				zinfo.external_attr = 0o40775 << 16  # drwxrwxr-x
				zinfo.external_attr |= 0x10  # MS-DOS directory flag
			else:
				zinfo.external_attr = 0o600 << 16  # ?rw-------
		else:
			zinfo = zinfo_or_arcname

		# repro-zipfile ADDED ##
		# Overwrite date_time and extrnal_attr (permissions mode)
		zinfo = copy(zinfo)
		zinfo.date_time = date_time()
		if zinfo.is_dir():
			zinfo.external_attr = (0o40000 | dir_mode()) << 16
			zinfo.external_attr |= 0x10  # MS-DOS directory flag
		else:
			zinfo.external_attr = file_mode() << 16
		#########################

		if not self.fp:
			raise ValueError("Attempt to write to ZIP archive that was already closed")
		if self._writing:
			raise ValueError(
				"Can't write to ZIP archive while an open writing handle exists."
			)

		if compress_type is not None:
			zinfo.compress_type = compress_type

		if compresslevel is not None:
			zinfo._compresslevel = compresslevel

		zinfo.file_size = len(data)  # Uncompressed size
		with self._lock:
			with self.open(zinfo, mode="w") as dest:
				dest.write(data)



import os
import sys
import unittest
from os.path import abspath, dirname

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)

from glossary_v2_test import TestGlossaryBase


class TestGlossaryCSV(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"100-en-de-v4.csv": "2890fb3e",
				"100-en-fa.csv": "eb8b0474",
				"100-en-fa-semicolon.csv": "b3f04599",
				"100-ja-en.csv": "7af18cf3",
			},
		)
		os.environ["CALC_FILE_SIZE"] = "1"

	def convert_txt_csv(self, fname, fname2, **convertArgs):
		self.convert(
			f"{fname}.txt",
			f"{fname}-2.csv",
			compareText=f"{fname2}.csv",
			**convertArgs,
		)

	def convert_csv_txt(self, fname, fname2, **convertArgs):
		self.convert(
			f"{fname}.csv",
			f"{fname}-2.txt",
			compareText=f"{fname2}.txt",
			**convertArgs,
		)

	def test_convert_txt_csv_1(self):
		self.convert_txt_csv("100-en-fa", "100-en-fa")

	def test_convert_txt_csv_2(self):
		self.convert_txt_csv("100-en-de-v4", "100-en-de-v4")

	def test_convert_txt_csv_3(self):
		self.convert_txt_csv("100-ja-en", "100-ja-en")

	def test_convert_txt_csv_4(self):
		self.convert_txt_csv(
			"100-en-fa",
			"100-en-fa-semicolon",
			writeOptions={"delimiter": ";"},
		)

	def test_convert_csv_txt_1(self):
		self.convert_csv_txt(
			"100-en-fa",
			"100-en-fa",
			infoOverride={"input_file_size": None},
		)

	def test_convert_csv_txt_2(self):
		self.convert_csv_txt(
			"100-en-de-v4",
			"100-en-de-v4",
		)

	def test_convert_csv_txt_3(self):
		self.convert_csv_txt(
			"100-ja-en",
			"100-ja-en",
			infoOverride={"input_file_size": None},
		)

	def test_convert_txt_csv_5(self):
		self.convert_csv_txt(
			"100-en-fa-semicolon",
			"100-en-fa",
			readOptions={"delimiter": ";"},
			infoOverride={"input_file_size": None},
		)


if __name__ == "__main__":
	unittest.main()



# -*- coding: utf-8 -*-
from __future__ import annotations

from typing import TYPE_CHECKING

from .reader import Reader

if TYPE_CHECKING:
	from pyglossary.option import Option

__all__ = [
	"Reader",
	"description",
	"enable",
	"extensionCreate",
	"extensions",
	"kind",
	"lname",
	"name",
	"optionsProp",
	"singleFile",
	"website",
	"wiki",
]

enable = True
lname = "makindo_medical"
name = "MakindoMedical"
description = "Makindo Medical Reference (SQLite3)"
extensions = ()
extensionCreate = ".db"
singleFile = True
kind = "binary"
wiki = ""
website = (
	"https://www.makindo.co.uk/topics/_index.php",
	"Makindo.co.uk Comprehensive Medical Encyclopedia",
)
optionsProp: dict[str, Option] = {}



# -*- coding: utf-8 -*-
from __future__ import annotations

from typing import TYPE_CHECKING

from pyglossary.core import log

if TYPE_CHECKING:
	from collections.abc import Generator

	from pyglossary.glossary_types import EntryType, WriterGlossaryType

__all__ = ["Writer"]


class Writer:
	def __init__(self, glos: WriterGlossaryType) -> None:
		self._glos = glos
		self._filename = ""

	def open(self, filename: str) -> None:
		self._filename = filename

	def write(self) -> Generator[None, EntryType, None]:
		glos = self._glos
		# filename = self._filename
		# log.info(f"some useful message")
		while True:
			entry = yield
			if entry is None:
				break
			if entry.isData():
				# can save it with entry.save(directory)
				continue
			# term = entry.s_term
			# defi = entry.defi
			# here write word and defi to the output file (depending on
			# your format)
		# here read info from Glossaey object
		name = glos.getInfo("name")
		desc = glos.getInfo("description")
		author = glos.author
		copyright_ = glos.getInfo("copyright")
		# if an info key doesn't exist, getInfo returns empty string
		# now write info to the output file (depending on your output format)
		log.info(f"{name=}")
		log.info(f"{desc=}")
		log.info(f"{author=}")
		log.info(f"{copyright_=}")

	def finish(self) -> None:
		self._filename = ""



'\nExports function list_versions() for unicode version level support.\n\nThis code generated by wcwidth/bin/update-tables.py on 2025-09-15 16:57:50 UTC.\n'
def list_versions():'\n\tReturn Unicode version levels supported by this module release.\n\n\tAny of the version strings returned may be used as keyword argument\n\t``unicode_version`` to the ``wcwidth()`` family of functions.\n\n\t:returns: Supported Unicode version numbers in ascending sorted order.\n\t:rtype: list[str]\n\t';return'4.1.0','5.0.0','5.1.0','5.2.0','6.0.0','6.1.0','6.2.0','6.3.0','7.0.0','8.0.0','9.0.0','10.0.0','11.0.0','12.0.0','12.1.0','13.0.0','14.0.0','15.0.0','15.1.0','16.0.0','17.0.0'


from __future__ import annotations

import os
import sys
import unittest
from os.path import abspath, dirname, join

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)

from glossary_v2_test import TestGlossaryBase

from pyglossary.glossary_v2 import ConvertArgs, Glossary


class TestGlossaryAppleDictBin(TestGlossaryBase):
	def setUp(self):
		if os.getenv("SKIP_TEST_MISSING_DEP"):
			try:
				import biplist  # noqa: F401
			except ImportError:
				self.skipTest("skipping module due to missing dependency: biplist")
		TestGlossaryBase.setUp(self)

	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		hashDict = {
			"appledict-bin/002-simple.txt": "32a1dbc4",
			"appledict-bin/002-simple.txt_res/style.css": "a83210cb",
			"appledict-bin/006-en-oxfjord_v10.11_c2_t3.txt": "2d3844bf",
			"appledict-bin/006-en-oxfjord_v10.11_c2_t3.txt_res/style.css": "14c3cf2c",
		}
		self.dataFileCRC32.update(hashDict)

		self.addDirCRC32(
			"appledict-bin/002-simple.dictionary",
			{
				"Contents/Info.plist": "fa73dd65",
				"Contents/Body.data": "3c073986",
				"Contents/DefaultStyle.css": "a83210cb",
				"Contents/EntryID.data": "37305249",
				"Contents/EntryID.index": "8c30a3fa",
				"Contents/Images/_internal_dictionary.png": "da4d4eb1",
				"Contents/KeyText.data": "aefe15e0",
				"Contents/KeyText.index": "b723c5b2",
				"Contents/MyDictionary.xsl": "023de1ea",
				"Contents/MyDictionary_prefs.html": "09a9f6e9",
			},
		)
		self.addDirCRC32(
			"appledict-bin/006-en-oxfjord_v10.11_c2_t3.dictionary",
			{
				"Contents/Info.plist": "328abb6f",
				"Contents/Resources/Body.data": "03fe72e8",
				"Contents/Resources/DefaultStyle.css": "c243b56a",
				"Contents/Resources/EntryID.data": "d31adec1",
				"Contents/Resources/EntryID.index": "6eea272c",
				"Contents/Resources/KeyText.data": "d4417c62",
				"Contents/Resources/KeyText.index": "59f9ab67",
			},
		)

	def convert_appledict_binary_to_txt(
		self,
		baseName: str,
		files: list[str],
		html_full: bool = False,
		resFiles: dict[str, str] | None = None,
	):
		if resFiles is None:
			resFiles = {}
		self.glos = Glossary()
		inputDirPath = self.downloadDir(
			f"appledict-bin/{baseName}.dictionary",
			files,
		)
		outputFilePath = self.newTempFilePath(f"{baseName}.txt")
		expectedOutputFilePath = self.downloadFile(
			f"appledict-bin/{baseName}.txt",
		)
		expectedStylePath = self.downloadFile(
			f"appledict-bin/{baseName}.txt_res/style.css",
		)

		result = self.glos.convert(
			ConvertArgs(
				inputFilename=inputDirPath,
				outputFilename=outputFilePath,
				inputFormat="AppleDictBin",
				outputFormat="Tabfile",
				readOptions={
					"html_full": html_full,
				},
			)
		)
		self.assertIsNotNone(result)
		self.assertEqual(result, outputFilePath)

		self.compareTextFiles(
			outputFilePath,
			expectedOutputFilePath,
		)
		self.compareTextFiles(
			join(outputFilePath + "_res", "style.css"),
			expectedStylePath,
		)
		for relPath, inputRelPath in resFiles.items():
			self.compareBinaryFiles(
				join(outputFilePath + "_res", relPath),
				join(inputDirPath, inputRelPath),
			)

	def test_appledict_binary_to_txt_0(self):
		baseName = "002-simple"
		files = [
			"Contents/Body.data",
			"Contents/DefaultStyle.css",
			"Contents/EntryID.data",
			"Contents/EntryID.index",
			"Contents/Images/_internal_dictionary.png",
			"Contents/Info.plist",
			"Contents/KeyText.data",
			"Contents/KeyText.index",
			"Contents/MyDictionary.xsl",
			"Contents/MyDictionary_prefs.html",
		]
		internal = "Images/_internal_dictionary.png"
		resFiles = {
			internal: f"Contents/{internal}",
		}
		self.convert_appledict_binary_to_txt(baseName, files, resFiles=resFiles)

	def test_appledict_binary_to_txt_1(self):
		baseName = "006-en-oxfjord_v10.11_c2_t3"
		files = [
			"Contents/Info.plist",
			"Contents/Resources/Body.data",
			"Contents/Resources/DefaultStyle.css",
			"Contents/Resources/EntryID.data",
			"Contents/Resources/EntryID.index",
			"Contents/Resources/KeyText.data",
			"Contents/Resources/KeyText.index",
		]
		self.convert_appledict_binary_to_txt(baseName, files)


if __name__ == "__main__":
	unittest.main()



"""
Functions that read and write gzipped files.

The user of the file doesn't have to worry about the compression,
but random access is not allowed.
"""

# based on Andrew Kuchling's minigzip.py distributed with the zlib module

try:
	# Python < 3.14
	from _compression import BaseStream, DecompressReader  # noqa: PLC2701
except ModuleNotFoundError:
	# Python 3.14
	from compression._common._streams import BaseStream, DecompressReader  # noqa: PLC2701

import builtins
import io
import logging
import os
import struct
import time
import zlib
from typing import IO

__all__ = ["GzipFile"]

log = logging.getLogger("root")

_FTEXT, FHCRC, FEXTRA, FNAME, FCOMMENT = 1, 2, 4, 8, 16

READ, WRITE = 1, 2

_COMPRESS_LEVEL_FAST = 1
_COMPRESS_LEVEL_BEST = 9


def write32u(output: IO[bytes], value: int) -> None:
	# The L format writes the bit pattern correctly whether signed
	# or unsigned.
	output.write(struct.pack("<L", value))


class _PaddedFile:
	"""
	Minimal read-only file object that prepends a string to the contents
	of an actual file. Shouldn't be used outside of gzip.py, as it lacks
	essential functionality.
	"""

	def __init__(self, f: IO[bytes], prepend: bytes = b"") -> None:
		self._buffer = prepend
		self._length = len(prepend)
		self.file = f
		self._read = 0

	def read(self, size: int) -> bytes:
		if self._read is None:
			return self.file.read(size)
		if self._read + size <= self._length:
			read = self._read
			self._read += size
			return self._buffer[read : self._read]

		read = self._read
		self._read = None
		return self._buffer[read:] + self.file.read(
			size - self._length + read,
		)

	def prepend(self, prepend: bytes = b"") -> None:
		if self._read is None:
			self._buffer = prepend
		else:  # Assume data was read since the last prepend() call
			self._read -= len(prepend)
			return
		self._length = len(self._buffer)
		self._read = 0

	def seek(self, off: int) -> int:
		self._read = None
		self._buffer = None
		return self.file.seek(off)

	def seekable(self) -> bool:  # noqa: PLR6301
		# to comply with io.IOBase
		return True  # Allows fast-forwarding even in unseekable streams


class BadGzipFile(OSError):
	"""Exception raised in some cases for invalid gzip files."""


class GzipFile(BaseStream):
	"""
	The GzipFile class simulates most of the methods of a file object with
	the exception of the truncate() method.

	This class only supports opening files in binary mode. If you need to open a
	compressed file in text mode, use the gzip.open() function.

	"""

	# Overridden with internal file object to be closed, if only a filename
	# is passed in
	myfileobj = None

	def __init__(
		self,
		filename: str | None = None,
		mode: str | None = None,
		compresslevel: int | None = _COMPRESS_LEVEL_BEST,
		fileobj: IO[bytes] | None = None,
		mtime: int | None = None,
	) -> None:
		"""
		Constructor for the GzipFile class.

		At least one of fileobj and filename must be given a
		non-trivial value.

		The new class instance is based on fileobj, which can be a regular
		file, an io.BytesIO object, or any other object which simulates a file.
		It defaults to None, in which case filename is opened to provide
		a file object.

		When fileobj is not None, the filename argument is only used to be
		included in the gzip file header, which may include the original
		filename of the uncompressed file.  It defaults to the filename of
		fileobj, if discernible; otherwise, it defaults to the empty string,
		and in this case the original filename is not included in the header.

		The mode argument can be any of 'r', 'rb', 'a', 'ab', 'w', 'wb', 'x', or
		'xb' depending on whether the file will be read or written.  The default
		is the mode of fileobj if discernible; otherwise, the default is 'rb'.
		A mode of 'r' is equivalent to one of 'rb', and similarly for 'w' and
		'wb', 'a' and 'ab', and 'x' and 'xb'.

		The compresslevel argument is an integer from 0 to 9 controlling the
		level of compression; 1 is fastest and produces the least compression,
		and 9 is slowest and produces the most compression. 0 is no compression
		at all. The default is 9.

		The mtime argument is an optional numeric timestamp to be written
		to the last modification time field in the stream when compressing.
		If omitted or None, the current time is used.

		"""
		if mode and ("t" in mode or "U" in mode):
			raise ValueError(f"Invalid mode: {mode!r}")
		if mode and "b" not in mode:
			mode += "b"
		if fileobj is None:
			fileobj = self.myfileobj = builtins.open(filename, mode or "rb")
		if filename is None:
			filename = getattr(fileobj, "name", "")
			if not isinstance(filename, (str, bytes)):
				filename = ""
		else:
			filename = os.fspath(filename)
		origmode = mode
		if mode is None:
			mode = getattr(fileobj, "mode", "rb")

		if mode.startswith("r"):
			self.mode = READ
			raw = _GzipReader(fileobj)
			self._buffer = io.BufferedReader(raw)
			self.name = filename

		elif mode.startswith(("w", "a", "x")):
			if origmode is None:
				import warnings

				warnings.warn(
					"GzipFile was opened for writing, but this will "
					"change in future Python releases.  "
					"Specify the mode argument for opening it for writing.",
					FutureWarning,
					2,
				)
			self.mode = WRITE
			self._init_write(filename)
			self.compress = zlib.compressobj(
				compresslevel, zlib.DEFLATED, -zlib.MAX_WBITS, zlib.DEF_MEM_LEVEL, 0
			)
			self._write_mtime = mtime
		else:
			raise ValueError(f"Invalid mode: {mode!r}")

		self.fileobj = fileobj

		if self.mode == WRITE:
			self._write_gzip_header(compresslevel)

	@property
	def filename(self) -> str:
		import warnings

		warnings.warn("use the name attribute", DeprecationWarning, 2)
		if self.mode == WRITE and self.name[-3:] != ".gz":
			return self.name + ".gz"
		return self.name

	@property
	def mtime(self) -> int | None:
		"""Last modification time read from stream, or None."""
		return self._buffer.raw._last_mtime

	def __repr__(self) -> str:
		s = repr(self.fileobj)
		return "<gzip " + s[1:-1] + " " + hex(id(self)) + ">"

	def _init_write(self, filename: str) -> None:
		self.name = filename
		self.crc = zlib.crc32(b"")
		self.size = 0
		self.offset = 0  # Current file offset for seek(), tell(), etc

	def _write_gzip_header(self, compresslevel: int) -> None:
		self.fileobj.write(b"\037\213")  # magic header
		self.fileobj.write(b"\010")  # compression method
		try:
			# RFC 1952 requires the FNAME field to be Latin-1. Do not
			# include filenames that cannot be represented that way.
			fname = os.path.basename(self.name)
			if not isinstance(fname, bytes):
				fname = fname.encode("latin-1")
			if fname.endswith(b".gz"):
				fname = fname[:-3]
		except UnicodeEncodeError:
			fname = b""
		flags = 0
		if fname:
			flags = FNAME
		self.fileobj.write(chr(flags).encode("latin-1"))
		mtime = self._write_mtime
		if mtime is None:
			mtime = time.time()
		write32u(self.fileobj, int(mtime))
		if compresslevel == _COMPRESS_LEVEL_BEST:
			xfl = b"\002"
		elif compresslevel == _COMPRESS_LEVEL_FAST:
			xfl = b"\004"
		else:
			xfl = b"\000"
		self.fileobj.write(xfl)
		self.fileobj.write(b"\377")
		if fname:
			self.fileobj.write(fname + b"\000")

	def write(self, data: bytes | bytearray | str) -> int:
		self._check_not_closed()
		if self.mode != WRITE:
			import errno

			raise OSError(errno.EBADF, "write() on read-only GzipFile object")

		if self.fileobj is None:
			raise ValueError("write() on closed GzipFile object")

		if isinstance(data, (bytes, bytearray)):
			length = len(data)
		else:
			# accept any data that supports the buffer protocol
			data = memoryview(data)
			length = data.nbytes

		if length > 0:
			self.fileobj.write(self.compress.compress(data))
			self.size += length
			self.crc = zlib.crc32(data, self.crc)
			self.offset += length

		return length

	def read(self, size: int = -1) -> int:
		self._check_not_closed()
		if self.mode != READ:
			import errno

			raise OSError(errno.EBADF, "read() on write-only GzipFile object")
		return self._buffer.read(size)

	def read1(self, size: int = -1) -> int:
		"""
		Implements BufferedIOBase.read1().

		Reads up to a buffer's worth of data if size is negative.
		"""
		self._check_not_closed()
		if self.mode != READ:
			import errno

			raise OSError(errno.EBADF, "read1() on write-only GzipFile object")

		if size < 0:
			size = io.DEFAULT_BUFFER_SIZE
		return self._buffer.read1(size)

	def peek(self, n: int) -> int:
		self._check_not_closed()
		if self.mode != READ:
			import errno

			raise OSError(errno.EBADF, "peek() on write-only GzipFile object")
		return self._buffer.peek(n)

	@property
	def closed(self) -> bool:
		return self.fileobj is None

	def close(self) -> None:
		fileobj = self.fileobj
		if fileobj is None:
			return
		self.fileobj = None
		try:
			if self.mode == WRITE:
				fileobj.write(self.compress.flush())
				write32u(fileobj, self.crc)
				# self.size may exceed 2 GiB, or even 4 GiB
				write32u(fileobj, self.size & 0xFFFFFFFF)
			elif self.mode == READ:
				self._buffer.close()
		finally:
			myfileobj = self.myfileobj
			if myfileobj:
				self.myfileobj = None
				myfileobj.close()

	def flush(self, zlib_mode: int = zlib.Z_SYNC_FLUSH) -> None:
		self._check_not_closed()
		if self.mode == WRITE:
			# Ensure the compressor's buffer is flushed
			self.fileobj.write(self.compress.flush(zlib_mode))
			self.fileobj.flush()

	def fileno(self) -> int:
		"""
		Invoke the underlying file object's fileno() method.

		This will raise AttributeError if the underlying file object
		doesn't support fileno().
		"""
		return self.fileobj.fileno()

	def rewind(self) -> None:
		"""
		Return the uncompressed stream file position indicator to the
		beginning of the file.
		"""
		if self.mode != READ:
			raise OSError("Can't rewind in write mode")
		self._buffer.seek(0)

	def readable(self) -> bool:
		return self.mode == READ

	def writable(self) -> bool:
		return self.mode == WRITE

	def seekable(self) -> bool:  # noqa: PLR6301
		return True

	def seek(self, offset: int, whence: int = io.SEEK_SET) -> int:
		if self.mode == WRITE:
			if whence != io.SEEK_SET:
				if whence == io.SEEK_CUR:
					offset = self.offset + offset
				else:
					raise ValueError("Seek from end not supported")
			if offset < self.offset:
				raise OSError("Negative seek in write mode")
			count = offset - self.offset
			chunk = b"\0" * 1024
			for _ in range(count // 1024):
				self.write(chunk)
			self.write(b"\0" * (count % 1024))
		elif self.mode == READ:
			self._check_not_closed()
			return self._buffer.seek(offset, whence)

		return self.offset

	def readline(self, size: int = -1) -> str:
		self._check_not_closed()
		return self._buffer.readline(size)


def _read_exact(fp: IO[bytes], n: int) -> bytes:
	"""
	Read exactly *n* bytes from `fp`.

	This method is required because fp may be unbuffered,
	i.e. return short reads.
	"""
	data = fp.read(n)
	while len(data) < n:
		b = fp.read(n - len(data))
		if not b:
			raise EOFError(
				"Compressed file ended before the end-of-stream marker was reached"
			)
		data += b
	return data


def _read_gzip_header(fp: IO[bytes]) -> bytes:
	"""
	Read a gzip header from `fp` and progress to the end of the header.

	Returns last mtime if header was present or None otherwise.
	"""
	magic = fp.read(2)
	if magic == b"":
		return None

	if magic != b"\037\213":
		raise BadGzipFile(f"Not a gzipped file ({magic!r})")

	(method, flag, last_mtime) = struct.unpack("<BBIxx", _read_exact(fp, 8))
	if method != 8:
		raise BadGzipFile("Unknown compression method")

	if flag & FEXTRA:
		# Read & discard the extra field, if present
		(extra_len,) = struct.unpack("<H", _read_exact(fp, 2))
		_read_exact(fp, extra_len)
	if flag & FNAME:
		# Read and discard a null-terminated string containing the filename
		while True:
			s = fp.read(1)
			if not s or s == b"\000":
				break
	if flag & FCOMMENT:
		# Read and discard a null-terminated string containing a comment
		while True:
			s = fp.read(1)
			if not s or s == b"\000":
				break
	if flag & FHCRC:
		_read_exact(fp, 2)  # Read & discard the 16-bit header CRC
	return last_mtime


class _GzipReader(DecompressReader):
	def __init__(self, fp: IO[bytes]) -> None:
		super().__init__(_PaddedFile(fp), zlib.decompressobj, wbits=-zlib.MAX_WBITS)
		# Set flag indicating start of a new member
		self._new_member = True
		self._last_mtime = None

	def _init_read(self) -> None:
		self._crc = zlib.crc32(b"")
		self._stream_size = 0  # Decompressed size of unconcatenated stream

	def _read_gzip_header(self) -> bool:
		last_mtime = _read_gzip_header(self._fp)
		if last_mtime is None:
			return False
		self._last_mtime = last_mtime
		return True

	def read(self, size: int = -1) -> bytes:
		if size < 0:
			return self.readall()
		# size=0 is special because decompress(max_length=0) is not supported
		if not size:
			return b""

		# For certain input data, a single
		# call to decompress() may not return
		# any data. In this case, retry until we get some data or reach EOF.
		while True:
			if self._decompressor.eof:
				# Ending case: we've come to the end of a member in the file,
				# so finish up this member, and read a new gzip header.
				# Check the CRC and file size, and set the flag so we read
				# a new member
				self._read_eof()
				self._new_member = True
				self._decompressor = self._decomp_factory(**self._decomp_args)

			if self._new_member:
				# If the _new_member flag is set, we have to
				# jump to the next member, if there is one.
				self._init_read()
				if not self._read_gzip_header():
					self._size = self._pos
					return b""
				self._new_member = False

			# Read a chunk of data from the file
			buf = self._fp.read(io.DEFAULT_BUFFER_SIZE)

			uncompress = self._decompressor.decompress(buf, size)
			if self._decompressor.unconsumed_tail != b"":
				self._fp.prepend(self._decompressor.unconsumed_tail)
			elif self._decompressor.unused_data != b"":
				# Prepend the already read bytes to the fileobj so they can
				# be seen by _read_eof() and _read_gzip_header()
				self._fp.prepend(self._decompressor.unused_data)

			if uncompress != b"":
				break
			if buf == b"":
				raise EOFError(
					"Compressed file ended before the end-of-stream marker was reached"
				)

		self._add_read_data(uncompress)
		self._pos += len(uncompress)
		return uncompress

	def _add_read_data(self, data: bytes) -> None:
		self._crc = zlib.crc32(data, self._crc)
		self._stream_size += len(data)

	def _read_eof(self) -> None:
		# We've read to the end of the file
		# We check that the computed CRC and size of the
		# uncompressed data matches the stored values.  Note that the size
		# stored is the true file size mod 2**32.
		crc32, isize = struct.unpack("<II", _read_exact(self._fp, 8))
		if crc32 != self._crc:
			log.warning(f"CRC check failed {crc32:#x} != `{self._crc:#x}")
		elif isize != (self._stream_size & 0xFFFFFFFF):
			raise BadGzipFile("Incorrect length of data produced")

		# Gzip files can be padded with zeroes and still have archives.
		# Consume all zero bytes and set the file position to the first
		# non-zero byte. See http://www.gzip.org/#faq8
		c = b"\x00"
		while c == b"\x00":
			c = self._fp.read(1)
		if c:
			self._fp.prepend(c)

	def _rewind(self) -> None:
		super()._rewind()
		self._new_member = True



# -*- coding: utf-8 -*-
# mypy: ignore-errors
#
# Copyright © 2025 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
# This file is part of PyGlossary project, https://github.com/ilius/pyglossary
#
# This program is a free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program. Or on Debian systems, from /usr/share/common-licenses/GPL
# If not, see <http://www.gnu.org/licenses/gpl.txt>.

from __future__ import annotations

import logging
from os.path import isfile, join
from typing import Any

from pyglossary.core import (
	appResDir,
	confJsonFile,
	dataDir,
	rootConfJsonFile,
)

from .config import configDefDict

__all__ = ["UIBase", "aboutText", "authors", "fread", "licenseText", "logo"]


def fread(path: str) -> str:
	with open(path, encoding="utf-8") as fp:
		return fp.read()


log = logging.getLogger("pyglossary")
logo = join(appResDir, "pyglossary.png")
aboutText = fread(join(dataDir, "about"))
licenseText = fread(join(dataDir, "_license-dialog"))
authors = fread(join(dataDir, "AUTHORS")).split("\n")

summary = (
	"A tool for converting dictionary files aka glossaries with"
	" various formats for different dictionary applications"
)


class UIBase:
	conflictingParams = [
		("sqlite", "direct"),
		("remove_html", "remove_html_all"),
	]

	def __init__(self, **_kwargs: Any) -> None:
		self.config = {}

	def progressInit(self, title: str) -> None:
		pass

	def progress(self, ratio: float, text: str = "") -> None:
		pass

	def progressEnd(self) -> None:
		self.progress(1.0)

	def loadConfig(
		self,
		user: bool = True,
		**options: Any,
	) -> None:
		from pyglossary.json_utils import jsonToData

		data = jsonToData(fread(rootConfJsonFile))
		assert isinstance(data, dict), f"{data=}"
		if user and isfile(confJsonFile):
			try:
				userData = jsonToData(fread(confJsonFile))
			except Exception:
				log.exception(
					f"error while loading user config file {confJsonFile!r}",
				)
			else:
				data.update(userData)

		for key in configDefDict:
			try:
				self.config[key] = data.pop(key)
			except KeyError:  # noqa: PERF203
				pass
		for key in data:
			log.warning(
				f"unknown config key {key!r}, you may edit {confJsonFile}"
				" file and remove this key",
			)

		for key, value in options.items():
			if key in configDefDict:
				self.config[key] = value

		log.setTimeEnable(self.config["log_time"])

		log.debug(f"loaded config: {self.config}")

	def saveConfig(self) -> None:
		from pyglossary.json_utils import dataToPrettyJson

		config = {}
		for key, option in configDefDict.items():
			if key not in self.config:
				log.warning(f"saveConfig: missing key {key!r}")
				continue
			value = self.config[key]
			if not option.validate(value):
				log.error(f"saveConfig: invalid {key}={value!r}")
				continue
			config[key] = value
		jsonStr = dataToPrettyJson(config)
		with open(confJsonFile, mode="w", encoding="utf-8") as file:
			file.write(jsonStr)
		log.info(f"saved {confJsonFile!r}")



# -*- coding: utf-8 -*-
# mypy: ignore-errors
#
# Copyright © 2025 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
#
# This program is a free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# You can get a copy of GNU General Public License along this program
# But you can always get it from http://www.gnu.org/licenses/gpl.txt
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

from __future__ import annotations

from typing import TYPE_CHECKING

from gi.repository import Gtk as gtk

from .browse import BrowseButton
from .utils import pack

if TYPE_CHECKING:
	from collections.abc import Callable

__all__ = ["InputFileBox", "OutputFileBox"]

_ = str  # later replace with translator function


class InputFileBox(gtk.Box):
	def __init__(
		self,
		entryChanged: Callable[[gtk.Entry], None],
		labelSizeGroup: gtk.SizeGroup = None,
		buttonSizeGroup: gtk.SizeGroup = None,
	) -> None:
		gtk.Box.__init__(self, orientation=gtk.Orientation.HORIZONTAL, spacing=3)
		label = gtk.Label(label=_("Input File:"))
		pack(self, label)
		label.set_property("xalign", 0)
		self.entry = gtk.Entry()
		pack(self, self.entry, 1, 1)
		button = BrowseButton(
			self.entry.set_text,
			label="Browse",
			actionSave=False,
			title="Select Input File",
		)
		pack(self, button)
		if labelSizeGroup:
			labelSizeGroup.add_widget(label)
		if buttonSizeGroup:
			buttonSizeGroup.add_widget(button)
		self.entry.connect("changed", entryChanged)

	def get_text(self) -> str:
		return self.entry.get_text()

	def set_text(self, text: str) -> None:
		return self.entry.set_text(text)


class OutputFileBox(gtk.Box):
	def __init__(
		self,
		entryChanged: Callable[[gtk.Entry], None],
		labelSizeGroup: gtk.SizeGroup = None,
		buttonSizeGroup: gtk.SizeGroup = None,
	) -> None:
		gtk.Box.__init__(self, orientation=gtk.Orientation.HORIZONTAL, spacing=3)
		label = gtk.Label(label=_("Ouput File:"))
		pack(self, label)
		label.set_property("xalign", 0)
		self.entry = gtk.Entry()
		pack(self, self.entry, 1, 1)
		button = BrowseButton(
			self.entry.set_text,
			label="Browse",
			actionSave=True,
			title="Select Output File",
		)
		pack(self, button)
		if labelSizeGroup:
			labelSizeGroup.add_widget(label)
		if buttonSizeGroup:
			buttonSizeGroup.add_widget(button)
		self.entry.connect("changed", entryChanged)

	def get_text(self) -> str:
		return self.entry.get_text()

	def set_text(self, text: str) -> None:
		return self.entry.set_text(text)



# -*- coding: utf-8 -*-
from __future__ import annotations

import re
from operator import itemgetter
from typing import TYPE_CHECKING
from zipfile import ZipFile

from pyglossary.json_utils import jsonToData

if TYPE_CHECKING:
	from collections.abc import Generator, Iterator
	from zipfile import ZipInfo

	from pyglossary.glossary_types import EntryType, ReaderGlossaryType

	from .types import (
		DefinitionObj,
		StructuredContent,
		YomichanDefinition,
	)

__all__ = ["Reader"]

FIELDS_TO_WRITE = ["title", "author", "description", "sourceLanguage", "targetLanguage"]
TERM_BASE_PATTERN = re.compile(r"term_bank_(\d+).json\Z")
BASE_BANK_PATTERN = re.compile(r".+_(meta_)?bank_(\d+).json\Z")


class Reader:
	useByteProgress = False
	compressions = ["zip"]

	def __init__(self, glos: ReaderGlossaryType) -> None:
		self._glos = glos
		self.clear()

	def clear(self) -> None:
		self._dictFile: ZipFile | None = None
		self._filename = ""
		self._isSequenced = False
		self._termBankFiles = []
		self._resourceFiles = []

	def open(self, filename: str) -> None:
		# TODO: Sanitize name
		self._filename = filename
		self._dictFile = ZipFile(filename)
		resourceFiles = []
		termFiles: list[tuple[int, ZipInfo]] = []
		for file in self._dictFile.filelist:
			match = TERM_BASE_PATTERN.match(file.filename)
			if match is not None:
				termFiles.append((int(match.group(1)), file))
				continue
			if file.filename == "index.json" or file.is_dir():
				continue
			# As currently there is no support for them
			if BASE_BANK_PATTERN.match(file.filename):
				continue
			resourceFiles.append(file)
		self._termBankFiles = [val for _, val in sorted(termFiles, key=itemgetter(0))]
		self._resourceFiles = resourceFiles
		self._readIndex()

	def close(self) -> None:
		if self._dictFile:
			self._dictFile.close()
		self.clear()

	def __len__(self) -> int:
		if self._dictFile is None:
			raise RuntimeError("Yomichan: len(reader) called while reader is not open")
		# TODO: how do I count real length??
		return len(self._termBankFiles)

	def __iter__(self) -> Iterator[EntryType]:
		if self._dictFile is None:
			raise RuntimeError("Yomichan: resources were read while reader is not open")
		for termBankFile in self._termBankFiles:
			yield from self._readTermBank(termBankFile.filename)
		yield from self._readUsedResources()

	def _readIndex(self) -> None:
		if self._dictFile is None:
			raise RuntimeError("Yomichan: resources were read while reader is not open")
		with self._dictFile.open("index.json") as indexFile:
			index = jsonToData(indexFile.read())
		if not isinstance(index, dict):
			raise RuntimeError("Yomichan: ill-formed yomichan dictionary")
		if index["format"] != 3:
			raise NotImplementedError(
				"Yomichan: supported only dictionaries of 3 version",
			)
		self._glos.setInfo("sourceLang", "ja")
		for c_field in FIELDS_TO_WRITE:
			value = index.get(c_field)
			if value is not None:
				self._glos.setInfo(c_field, value)
		self._isSequenced = index.get("isSequenced", False)

	def _readTermBank(self, termBankName: str) -> Generator[EntryType, None, None]:
		if self._dictFile is None:
			raise RuntimeError("Yomichan: resources were read while reader is not open")
		with self._dictFile.open(termBankName) as termBankFile:
			termBank = jsonToData(termBankFile.read())
		for item in termBank:
			term = item[0]
			if reading := item[1]:
				term = [term, reading]
			definition = _readDefinition(item[5])
			yield self._glos.newEntry(term, definition, defiFormat="h")

	def _readUsedResources(self) -> Generator[EntryType, None, None]:
		if self._dictFile is None:
			raise RuntimeError("Yomichan: resources were read while reader is not open")
		for file in self._resourceFiles:
			with self._dictFile.open(file.filename) as rawFile:
				data = rawFile.read()
			yield self._glos.newDataEntry(file.filename, data)


def _readSubStructuredContent(elem: StructuredContent) -> str:
	if isinstance(elem, str):
		return elem
	if isinstance(elem, list):
		return " ".join(map(_readSubStructuredContent, elem))
	styles = []
	additional_properties = [
		f' {key}="{val}"' for key, val in elem.get("data", {}).items()
	]
	if "style" in elem:
		styles = [f"{k}: {v};" for k, v in elem["style"].items()]
	for field in ("lang", "title", "alt", "href", "colSpan", "rowSpan"):
		if field not in elem:
			continue
		additional_properties.append(f' {field}="{elem.get(field)}"')

	# Tag processing
	# TODO: Process all tags?
	if elem["tag"] == "br":
		return f"<br{''.join(additional_properties)}>"
	if elem["tag"] == "img":
		for name in ("width", "height"):
			if name not in elem:
				continue
			if elem.get("sizeUnits", "px") == "em":
				styles.append(f"{name}: {elem.get(name)}em;")
			else:
				additional_properties.append(f" {name}={elem.get(name)}")
		if "verticalAlign" in elem:
			styles.append(f"vertical-align: {elem['verticalAlign']};")
		additional_properties.append(f' style="{"".join(styles)}"')
		additional_properties = "".join(additional_properties)
		return f'<img src="{elem["path"]}"{additional_properties}>'
	# General tags
	tag = elem["tag"]
	content = _readSubStructuredContent(elem.get("content", ""))
	additional_properties.append(f' style="{"".join(styles)}"')
	additional_properties = "".join(additional_properties)
	return f"<{tag}{additional_properties}>{content}</{tag}>"


def _readStructuredContent(elem: DefinitionObj) -> str:
	if elem["type"] == "text":
		return elem["text"]
	if elem["type"] == "structured-content":
		return _readSubStructuredContent(elem["content"])
	if elem["type"] == "image":
		properties = []
		for name in ["alt", "width", "height", "title"]:
			prop = elem.get(name)
			if prop is not None:
				properties.append(f' {name}="{prop}"')
		return f'<img src="{elem["path"]}"{"".join(properties)}>'
	raise RuntimeError("Ill-formed Yomichan dictionary")


def _readDefinition(definition: list[YomichanDefinition]) -> str:
	def _ParseDefinition(defi: YomichanDefinition) -> str:
		if isinstance(defi, str):
			return defi
		if isinstance(defi, dict):
			return _readStructuredContent(defi)
		raise NotImplementedError(f"Yomichan: unknown elem in definition: {defi}")

	return "\n".join(map(_ParseDefinition, definition))



# -*- coding: utf-8 -*-

import sys
import unittest
from os.path import abspath, dirname

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)


from pyglossary.glossary_v2 import Glossary
from pyglossary.plugins.ebook_kobo import Writer


class GetPrefixTest(unittest.TestCase):
	def case(self, word, prefix):
		glos = Glossary()
		w = Writer(glos)
		self.assertEqual(
			w.get_prefix(word),
			prefix,
		)

	def test_examples(self):
		# examples from https://pgaskin.net/dictutil/dicthtml/prefixes.html
		self.case("test", "te")
		self.case("a", "aa")
		self.case("Èe", "èe")
		self.case("multiple words", "mu")
		self.case("àççèñts", "àç")
		self.case("à", "àa")
		self.case("ç", "ça")
		self.case("", "11")
		self.case(" ", "11")
		self.case(" x", "xa")
		self.case(" 123", "11")
		self.case("x 23", "xa")
		self.case("д ", "д")
		self.case("дaд", "дa")
		self.case("未未", "未未")
		self.case("未", "未a")
		self.case("  未", "11")
		self.case(" 未", "未a")

	# the rest of test cases are from
	# https://github.com/pgaskin/dictutil/blob/master/kobodict/util_test.go

	def test_dicthtml_en(self):
		self.case("a-", "11")
		self.case("-an", "11")
		self.case("GB", "gb")

	def test_dicthtml_fr(self):
		self.case("ébahir", "éb")
		self.case("a1", "11")
		self.case("ô", "ôa")
		self.case("kébab", "ké")
		self.case("aérer", "aé")
		self.case("living-room", "li")

	# dicthtml-ja
	# Note, Kanji not currently implemented, so not testing (note, the logic
	# is in a separate function, anyways).
	# self.case("あ", "あ")
	# self.case("アークとう", "アー")

	def test_dictword_spaces(self):
		# generated by dictword-test: spaces
		self.case("  ", "11")
		self.case("   ", "11")
		self.case("\t\t", "11")
		self.case("\t\f\t", "11")
		self.case("x ", "xa")
		self.case(" xx", "xa")

		# generated by dictword-test: spaces where trim/prefix order matters
		self.case("  x", "11")
		self.case("  xy", "11")
		self.case("  xyz", "11")
		self.case("x z", "xa")

	def test_dictword_cyrillic(self):
		# generated by dictword-test: cyrillic
		self.case(" д", "д")
		self.case(" дд", "д")
		self.case("д", "д")
		self.case("aд", "aд")
		self.case("дa", "дa")
		self.case("aдa", "aд")

	def test_dictword_uppercase_accented(self):
		# generated by dictword-test: uppercase accented letters
		self.case("Ȅe", "ȅe")
		self.case("eȄ", "eȅ")
		self.case("Ȅ", "ȅa")
		self.case("Ȅ!", "11")

	def test_dictword_cjk(self):
		# generated by dictword-test: cjk
		self.case("x未", "x未")
		self.case("未x", "未x")
		self.case("xy未", "xy")
		self.case("还没", "还没")

	def test_dictword_misc(self):
		# generated by dictword-test: misc
		self.case("!", "11")
		self.case("!!", "11")
		self.case("!!!", "11")
		self.case("x!", "11")
		self.case("x!!", "11")
		self.case("xx!", "xx")
		self.case("xxx!", "xx")
		self.case("  !", "11")
		self.case(" !!", "11")
		self.case(" !!!", "11")
		self.case(" !", "11")
		self.case("  !!", "11")
		self.case("   !!!", "11")
		self.case(" x!", "xa")
		self.case(" x!!", "xa")
		self.case(" xx!", "xa")
		self.case(" xxx!", "xa")

	def test_synthetic(self):
		self.case("x\x00y", "xa")
		self.case("\x00xy", "11")


if __name__ == "__main__":
	unittest.main()



# -*- coding: utf-8 -*-
#
# Copyright © 2025 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
#
# based on https://github.com/abelcheung/types-lxml
# under Apache License, Version 2.0, January 2004
# http://www.apache.org/licenses/

from __future__ import annotations

import typing
from collections.abc import Mapping
from contextlib import AbstractContextManager as ContextManager
from typing import AnyStr, Literal, TypeAlias

from lxml.etree import QName, _Element  # noqa: PLC2701

__all__ = ["Element", "T_htmlfile"]

_TextArg: TypeAlias = str | bytes | QName
_TagName: TypeAlias = _TextArg


_OutputMethodArg = Literal[
	"html",
	"text",
	"xml",
	"HTML",
	"TEXT",
	"XML",
]


# Element type can not be a protocol or interface or even TypeAlias
# it's stupid!
# And now pyright complains at every usage of it:
# error: Variable not allowed in type expression (reportInvalidTypeForm)

Element = _Element


class IncrementalFileWriter(typing.Protocol):
	# def write_declaration(
	# 	self,
	# 	version: AnyStr | None = ...,
	# 	standalone: bool | None = ...,
	# 	doctype: AnyStr | None = ...,
	# ) -> None: ...

	# def write_doctype(
	# 	self,
	# 	doctype: AnyStr | None,
	# ) -> None: ...

	def write(
		self,
		*args: AnyStr | Element,
		with_tail: bool = ...,
		pretty_print: bool = ...,
		method: _OutputMethodArg | None = ...,
	) -> None: ...

	def flush(self) -> None: ...

	def method(
		self,
		method: _OutputMethodArg | None,
	) -> ContextManager[None]:
		raise NotImplementedError

	def element(
		self,
		tag: _TagName,
		attrib: Mapping[str, AnyStr] | None = ...,
		nsmap: dict[str | None, AnyStr] | None = ...,
		method: _OutputMethodArg | None = ...,
		**_extra: AnyStr,
	) -> ContextManager[None]:
		raise NotImplementedError


# class AsyncIncrementalFileWriter(typing.Protocol):
# 	async def write_declaration(
# 		self,
# 		version: AnyStr | None = ...,
# 		standalone: bool | None = ...,
# 		doctype: AnyStr | None = ...,
# 	) -> None: ...

# 	async def write_doctype(
# 		self,
# 		doctype: AnyStr | None,
# 	) -> None: ...

# 	async def write(
# 		self,
# 		*args: AnyStr | Element | None,
# 		with_tail: bool = ...,
# 		pretty_print: bool = ...,
# 		method: _OutputMethodArg | None = ...,
# 	) -> None: ...

# 	async def flush(self) -> None: ...

# 	def method(
# 		self,
# 		method: _OutputMethodArg | None,
# 	) -> AsyncContextManager[None]: ...

# 	def element(
# 		self,
# 		tag: _TagName,
# 		attrib: Mapping[str, AnyStr] | None = ...,
# 		nsmap: dict[str | None, AnyStr] | None = ...,
# 		method: _OutputMethodArg | None = ...,
# 		**_extra: AnyStr,
# 	) -> AsyncContextManager[None]: ...


class T_htmlfile(  # type: ignore # noqa: PGH003
	IncrementalFileWriter,
	ContextManager[IncrementalFileWriter],
	# AsyncIncrementalFileWriter,
	# AsyncContextManager[AsyncIncrementalFileWriter],
):
	pass


# typing.AsyncContextManager
# is generic version of contextlib.AbstractAsyncContextManager



# -*- coding: utf-8 -*-
# mypy: ignore-errors
#
# Copyright © 2025 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
#
# This program is a free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# You can get a copy of GNU General Public License along this program
# But you can always get it from http://www.gnu.org/licenses/gpl.txt
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

from __future__ import annotations

import logging
from typing import Any

from gi.repository import Gdk as gdk
from gi.repository import Gio as gio
from gi.repository import GLib as glib
from gi.repository import Gtk as gtk

from pyglossary.core import appResDir
from pyglossary.ui.base import UIBase

from .mainwin import MainWindow
from .utils import gtk_event_iteration_loop

__all__ = ["UI"]

log = logging.getLogger("pyglossary")

glib.set_prgname("PyGlossary (Gtk4)")
gtk.Window.set_default_icon_name("pyglossary")

# ~/.local/share/icons/hicolor/scalable/apps/pyglossary.svg must exist
# unless we call iconTheme.add_search_path with our res dir, and make sure
# this file exists: res/hicolor/scalable/apps/pyglossary.svg
gtk.IconTheme.get_for_display(gdk.Display.get_default()).add_search_path(appResDir)


class UI(UIBase, gtk.Application):
	def __init__(
		self,
		progressbar: bool = True,
	) -> None:
		UIBase.__init__(self)
		gtk.Application.__init__(
			self,
			application_id="com.github.ilius.pyglossary",
			flags=gio.ApplicationFlags.DEFAULT_FLAGS,
		)
		if progressbar:
			self.progressBar = gtk.ProgressBar()
			self.progressBar.set_fraction(0)
		else:
			self.progressBar = None
		self.runArgs = {}
		self.mainWindow: MainWindow | None = None

	def run(self, **kwargs: Any) -> None:
		self.runArgs = kwargs
		gtk.Application.run(self)

	def do_activate(self) -> None:
		self.mainWindow = MainWindow(
			ui=self,
			app=self,
			progressBar=self.progressBar,
		)
		self.mainWindow.run(**self.runArgs)

	def progressInit(self, title: str) -> None:
		self.progressTitle = title

	def progress(self, ratio: float, text: str = "") -> None:
		if self.mainWindow is None:
			return
		if not text:
			text = "%" + str(int(ratio * 100))
		text += " - " + self.progressTitle
		self.progressBar.set_fraction(ratio)
		# self.progressBar.set_text(text)  # not working
		self.mainWindow.status(text)
		gtk_event_iteration_loop()



import unittest

import lxml
from glossary_v2_test import TestGlossaryBase

if not lxml.__version__.startswith("5.3."):
	raise OSError(f"Found lxml=={lxml.__version__}, must use lxml==5.3")


class TestGlossaryJMdict(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"050-JMdict-English": "aec9ad8c",
				"050-JMdict-English-v3.txt": "6068b9a7",
			},
		)
		# os.environ["CALC_FILE_SIZE"] = "1"

	def convert_jmdict_txt(self, fname, fname2, **convertArgs):
		self.convert(
			fname,
			f"{fname}-2.txt",
			compareText=f"{fname2}.txt",
			inputFormat="JMDict",
			**convertArgs,
		)

	# with lxml==5.3.0, for "bword://{word}", `word` is not unicode-escaped by lxml
	# while lxml < 5.3.0 does escape these unicode characters
	# that's why 050-JMdict-English-v2 was updated to 050-JMdict-English-v3
	def test_convert_jmdict_txt_1(self):
		self.convert_jmdict_txt(
			"050-JMdict-English",
			"050-JMdict-English-v3",
		)


if __name__ == "__main__":
	unittest.main()



import os
import sys
import unittest
from os.path import abspath, dirname

rootDir = dirname(dirname(abspath(__file__)))
sys.path.insert(0, rootDir)

from glossary_v2_test import TestGlossaryBase


class TestGlossaryWiktextract(TestGlossaryBase):
	def __init__(self, *args, **kwargs):
		TestGlossaryBase.__init__(self, *args, **kwargs)

		self.dataFileCRC32.update(
			{
				"wiktextract/10-kaikki-fa-PlacesInIran.jsonl": "f7f4a92f",
				"wiktextract/10-kaikki-fa-pos-adv.jsonl": "2ddcbbbd",
				"wiktextract/03-kaikki-fa-selection.jsonl": "31223225",
				"wiktextract/03-kaikki-fa-selection-v3.txt": "9e6dcb38",
				"wiktextract/10-kaikki-fa-PlacesInIran-category-v3.txt": "da7ae87d",
				"wiktextract/10-kaikki-fa-PlacesInIran-v3.txt": "c904b376",
				"wiktextract/10-kaikki-fa-pos-adv-v3.txt": "6bd35c97",
				"wiktextract/10-kaikki-fa-pos-adv-word_title-v3.txt": "3217ea15",
			},
		)
		os.environ["CALC_FILE_SIZE"] = "1"

	def convert_jsonl_txt(self, fname, fname2, **convertArgs):
		self.convert(
			f"wiktextract/{fname}.jsonl",
			f"{fname}-2.txt",
			compareText=f"wiktextract/{fname2}.txt",
			infoOverride={
				# without this, glos name would become f"wiktextract__{fname}.jsonl"
				"name": f"{fname}.jsonl",
			},
			**convertArgs,
		)

	def test_convert_jsonl_txt_1(self):
		self.convert_jsonl_txt(
			"10-kaikki-fa-PlacesInIran",
			"10-kaikki-fa-PlacesInIran-v3",
		)

	def test_convert_jsonl_txt_1_cats(self):
		self.convert_jsonl_txt(
			"10-kaikki-fa-PlacesInIran",
			"10-kaikki-fa-PlacesInIran-category-v3",
			readOptions={
				"categories": True,
			},
		)

	def test_convert_jsonl_txt_2(self):
		self.convert_jsonl_txt(
			"10-kaikki-fa-pos-adv",
			"10-kaikki-fa-pos-adv-v3",
		)

	def test_convert_jsonl_txt_2_word_title(self):
		self.convert_jsonl_txt(
			"10-kaikki-fa-pos-adv",
			"10-kaikki-fa-pos-adv-word_title-v3",
			readOptions={
				"word_title": True,
			},
		)

	def test_convert_jsonl_txt_3(self):
		self.convert_jsonl_txt(
			"03-kaikki-fa-selection",
			"03-kaikki-fa-selection-v3",
		)
		# testing these features
		# "antonyms" in sense
		# "topics" in sense
		# "form_of" in sense


if __name__ == "__main__":
	unittest.main()
